/******************************************************************************
 *                                                                            *
 *  Hoplite (c) Frans Ekman 2003                                              *
 *                                                                            *
 *  This takes care of almost everything related to the engines input/output  *
 *  in a way defined in the CHESS ENGINE COMMUNICATION PROTOCOLL.             *
 *                                                                            *
 *  modified 15.3.2004                                                        *
 *                                                                            *
 *****************************************************************************/
#include "engine.h"


void initiallizeEngine(TEngine *engine)
{
  engine->protover = 1;
  engine->go = FALSE;
  engine->computerColor = BLACK;
  engine->printThinkingOutput = FALSE;
  engine->time_tc = 60;
  engine->time_inc = 0;
  engine->time_mps = 0;
  engine->time_left = 60;  
  engine->time_left_opponent = 60;
  setStart(&(engine->game));
}


void playInXboard()
{
  char input[255];
  int inputValue;
  TEngine hopliteEngine;
  TEngine *hoplite;
  PMoveList list;
  PMove move;
  TMove compMove;
  int legal;
  int gameOver = FALSE;
  double max_time, min_time;
  int i;

  printf("feature done=0\n");
  fflush(stdout);

  generateMask();
  hoplite = &hopliteEngine;
  initiallizeEngine(hoplite);

  scanf("%s", input);
  if (strcmp(input, "protover")==0) {
    scanf("%d", &inputValue);
    hoplite->protover = inputValue;
    generateZobristKeys();   
    initiallize_openings();
    clearHash();
    initiallizeHistoryMoves();
    clearKillerMoves();
    printf("feature setboard=1 sigint=0 sigterm=0 colors=1 myname=\"Hoplite 2.1.1\" done=1\n");
    fflush(stdout);
    scanf("%s", input);
  } else {
    generateZobristKeys(); 
    initiallize_openings();
    clearHash();
    initiallizeHistoryMoves();
    clearKillerMoves();
  }


  while (1) {

    if (strcmp(input, "quit")==0) {
      return;
    } else if (strcmp(input, "accepted")==0) {
      scanf("%s", input);
    } else if (strcmp(input, "rejected")==0) {
        scanf("%s", input);
    } else if (strcmp(input, "new")==0) {
      command_new(hoplite);
      clearKillerMoves();
      clearHash();
      gameOver = FALSE;
    } else if (strcmp(input, "force")==0) {
      hoplite->go = FALSE;
    } else if (strcmp(input, "go")==0) {
      hoplite->go = TRUE;
      /*if (hoplite->game.currentPosition.whitesMove) 
        hoplite->computerColor = WHITE;
      else 
      hoplite->computerColor = BLACK;*/
    } else if (strcmp(input, "white")==0) {
      hoplite->computerColor = WHITE;
    } else if (strcmp(input, "black")==0) {
      hoplite->computerColor = BLACK;
    } else if (strcmp(input, "black")==0) {
      hoplite->computerColor = BLACK;
    } else if (strcmp(input, "level")==0) {
      scanf("%d", &inputValue);
      hoplite->time_mps = inputValue;
      scanf("%d", &inputValue);
      if (getc(stdin)==':') {
        hoplite->time_tc = 60 * inputValue;
        scanf("%d", &inputValue);
        hoplite->time_tc += inputValue;  
        hoplite->time_left = hoplite->time_tc;
        hoplite->time_left_opponent = hoplite->time_tc;
      }
      else
        hoplite->time_tc = 60 * inputValue;
      scanf("%d", &inputValue);
      hoplite->time_inc = inputValue;
    } else if (strcmp(input, "time")==0) {
      scanf("%d", &inputValue);
      hoplite->time_left = ((double)inputValue)/100;
    } else if (strcmp(input, "otim")==0) {
      scanf("%d", &inputValue);
      hoplite->time_left_opponent = ((double)inputValue)/100;
    } else if (strcmp(input, "post")==0) {
      hoplite->printThinkingOutput = TRUE;
    } else if (strcmp(input, "nopost")==0) {
      hoplite->printThinkingOutput = FALSE;
    }
/*
  -------------------------------
 |  Handle the setboard command  |
  -------------------------------
*/
    else if (strncmp(input, "setboard", 8)==0) {
      getc(stdin);
      hoplite->game.firstFree = 0;
      for (i=0; i<255; i++) {
        input[i] = getc(stdin);
        if (input[i] == '\0' || input[i] == '\n' || input[i] == EOF)
          break;
      }
      setPosition(&(hoplite->game.currentPosition), input);
      hoplite->game.currentPosition.knowIfCheck = FALSE;
      evaluateCheck(&(hoplite->game.currentPosition));
    }     
/*
  ---------------------
 |  Handle a usermove  |
  ---------------------
*/   
    else if (isalpha(input[0]) && isdigit(input[1]) &&
             isalpha(input[2]) && isdigit(input[3]) && !gameOver && 
             (hoplite->game.currentPosition.whitesMove 
              != hoplite->computerColor)) {
      legal = FALSE;
      move = createMove(&(hoplite->game.currentPosition), input);
      list = generateMoves(&(hoplite->game.currentPosition));
      for (i=0; i<list->firstFree; i++) { 
        if (move->x1 == list->moves[i]->x1 &&
            move->x2 == list->moves[i]->x2 &&
            move->y1 == list->moves[i]->y1 &&
            move->y2 == list->moves[i]->y2 &&
            move->promotion == list->moves[i]->promotion) {
          makeMove(&(hoplite->game), move);
          legal = TRUE;
          break;
        }
      }
      freeMoveList(list);
      if (!legal) {
        printf("Illegal move: ");
        p_move(move, stdout);
        printf("\n");
        fflush(stdout);
      }
      free(move);
      CHECK_IF_THE_GAME_HAS_ENDED;
    }
    
/*
  -------------------------------------------------
 |  Now check if it is the computers turn to move  |
  -------------------------------------------------
*/    
    if (hoplite->game.currentPosition.whitesMove == hoplite->computerColor &&
        hoplite->go && !gameOver) {

      move = NULL;
      move = getMoveFromOpenings(&(hoplite->game));
      if (move != NULL) {
        compMove = *move;
        makeMove(&(hoplite->game), &compMove);
      } else {
        list = generateMoves(&(hoplite->game.currentPosition));

        if (hoplite->time_left > 2 * hoplite->time_inc) {
          max_time = ((hoplite->time_left) / 30) + hoplite->time_inc;
          min_time = ((hoplite->time_left) / 60) + (hoplite->time_inc)/2;
        } else {
          max_time = (hoplite->time_left) / 30;
          min_time = (hoplite->time_left) / 60;
        }

        if (list->firstFree > 1)
          compMove = iterate(&(hoplite->game), min_time, max_time);
        else 
          compMove = *(list->moves[0]);
        if (compMove.promotion == 'F') {
          compMove = *(list->moves[0]);
        }
        makeMove(&(hoplite->game), &compMove);
        freeMoveList(list);
      }

      printf("move ");
      p_move(&compMove, stdout);
      printf("\n");
      fflush(stdout);
      
      CHECK_IF_THE_GAME_HAS_ENDED; 

      clearHash();
      clearHistoryMoves();
    }
    
    /**** read next input ****/
    scanf("%s", input);   
  }
}


void command_new(TEngine *engine)
{
  setStart(&(engine->game));
  engine->go = TRUE;
  engine->computerColor = BLACK;
  engine->time_left = engine->time_tc;
}




















