/******************************************************************************
 *
 *  Hoplite (c) Frans Ekman
 *
 *  Evaluation function implementation 
 *
 *  modified 24.2.2004
 *
 *****************************************************************************/

#include "evaluate.h"

int evaluations;

int evaluateWhiteDevelopment(PPosition position)
{
  int score = 0;
  if (position->board[4][1] == 'P') score += UNDEVELOPED_CENTERPAWN;
  if (position->board[3][1] == 'P') score += UNDEVELOPED_CENTERPAWN;
  if (position->board[6][0] == 'N') score += UNDEVELOPED_K_KNIGHT;
  if (position->board[5][0] == 'B') score += UNDEVELOPED_K_BISHOP;
  if (position->board[2][0] == 'B') score += UNDEVELOPED_Q_BISHOP;
  if (position->board[1][0] == 'N') score += UNDEVELOPED_Q_KNIGHT;
  return score;
}


int evaluateBlackDevelopment(PPosition position)
{
  int score = 0;
  if (position->board[4][6] == 'p') score += UNDEVELOPED_CENTERPAWN;
  if (position->board[3][6] == 'p') score += UNDEVELOPED_CENTERPAWN;
  if (position->board[6][7] == 'n') score += UNDEVELOPED_K_KNIGHT;
  if (position->board[5][7] == 'b') score += UNDEVELOPED_K_BISHOP;
  if (position->board[2][7] == 'b') score += UNDEVELOPED_Q_BISHOP;
  if (position->board[1][7] == 'n') score += UNDEVELOPED_Q_KNIGHT;
  return score;
}


int pawnScore_w(PPosition position, int x, int y)
{
  int score = -x * x + 8 * x + y + PAWN_VALUE;
  if (x > 1 && x < 5) {
    if (y > 1) score += PAWN_ADV_1SQUARE;
    if (x > 2 && y > 2) score += PAWN_ADV_2SQUARE;
  }
  return score;
}

int pawnScore_b(PPosition position, int x, int y)
{
  int score = -x * x + 8 * x + (7-y) + PAWN_VALUE;
  if (x > 1 && x < 5) {
    if (y < 6) score += PAWN_ADV_1SQUARE;
    if (x > 2 && y < 5) score += PAWN_ADV_2SQUARE;
  }
  return score;
}


int knightScore_w(PPosition position, int x, int y)
{
  return (3*PLACE_W + KNIGHT_VALUE);
}


int knightScore_b(PPosition position, int x, int y)
{
  return (3*PLACE_B + KNIGHT_VALUE);
}


int bishopScore_w(PPosition position, int x, int y)
{
  return BISHOP_VALUE;
}


int bishopScore_b(PPosition position, int x, int y)
{
  return BISHOP_VALUE;
}

int rookScore_w(PPosition position, int x, int y)
{
  return ROOK_VALUE;
}


int rookScore_b(PPosition position, int x, int y)
{
  return ROOK_VALUE;
}


int queenScore_w(PPosition position, int x, int y)
{
  return QUEEN_VALUE;
}


int queenScore_b(PPosition position, int x, int y)
{
  return QUEEN_VALUE;
}


int evaluate_position(PGame game)
{
  EState state = normal;
  int white_pieces[5] = {0,0,0,0,0};
  int black_pieces[5] = {0,0,0,0,0};
  int white_pawns_file[8] = {0,0,0,0,0,0,0,0};
  int black_pawns_file[8] = {0,0,0,0,0,0,0,0};
  int white_most_backward_pawn[8] = {6,6,6,6,6,6,6,6};
  int black_most_backward_pawn[8] = {1,1,1,1,1,1,1,1};
  int white_score = 0;
  int black_score = 0;
  int i, j;
  int was_passed_pawn = FALSE;
  char piece;
  PPosition position = &game->currentPosition;
  int whiteDevelopment = evaluateWhiteDevelopment(position);
  int blackDevelopment = evaluateBlackDevelopment(position);
  int whitePawns_on_w=0,whitePawns_on_b=0,blackPawns_on_w=0,blackPawns_on_b=0;

  evaluations++;

  white_score += whiteDevelopment;
  black_score += blackDevelopment;
  // check what material and pawns on files
  for (i=0; i<8; i++) {
    for (j=0; j<8; j++) {
      piece = position->board[i][j];
      if (piece == ' ') continue;
      else if (piece == 'P') {
        if ((i+j)%2)
          whitePawns_on_w++;
        else whitePawns_on_b++;
        white_pieces[INDEX_PAWN]++;
        white_pawns_file[i]++;
        if (j < white_most_backward_pawn[i]) 
          white_most_backward_pawn[i] = j;
      }
      else if (piece == 'p') {
        if ((i+j)%2)
          blackPawns_on_w++;
        else blackPawns_on_b++;
        black_pieces[INDEX_PAWN]++;
        black_pawns_file[i]++;
        if (j > black_most_backward_pawn[i]) 
          black_most_backward_pawn[i] = j;
      } 
      else if (piece == 'N') white_pieces[INDEX_KNIGHT]++; 
      else if (piece == 'n') black_pieces[INDEX_KNIGHT]++;  
      else if (piece == 'B') white_pieces[INDEX_BISHOP]++;
      else if (piece == 'b') black_pieces[INDEX_BISHOP]++;  
      else if (piece == 'R') white_pieces[INDEX_ROOK]++; 
      else if (piece == 'r') black_pieces[INDEX_ROOK]++;  
      else if (piece == 'Q') white_pieces[INDEX_QUEEN]++; 
      else if (piece == 'q') black_pieces[INDEX_QUEEN]++;  
    }
  }

  // check if it's an ending
  if (white_pieces[INDEX_QUEEN]==0 && black_pieces[INDEX_QUEEN]==0) {
    if (white_pieces[INDEX_ROOK]<=1 && black_pieces[INDEX_ROOK]<=1)
      state = endgame;
    else if ((white_pieces[INDEX_KNIGHT] + white_pieces[INDEX_BISHOP] +
             white_pieces[INDEX_ROOK]) <= 3 &&
             (black_pieces[INDEX_KNIGHT] + black_pieces[INDEX_BISHOP] +
              black_pieces[INDEX_ROOK]) <= 3)
      state = endgame;
  }
  else if ((white_pieces[INDEX_QUEEN] + white_pieces[INDEX_ROOK]) <= 1 &&
           (black_pieces[INDEX_QUEEN] + black_pieces[INDEX_ROOK]) <= 1) {
    if ((white_pieces[INDEX_KNIGHT] + white_pieces[INDEX_BISHOP]) <= 1 &&
        (black_pieces[INDEX_KNIGHT] + black_pieces[INDEX_BISHOP]) <= 1)
      state = endgame;
  }

  // Check if it is a special ending
  if (state == endgame)
  {
    // Black has only a king
    if (black_pieces[INDEX_PAWN] == 0 && black_pieces[INDEX_KNIGHT] == 0 &&
        black_pieces[INDEX_BISHOP] == 0 && black_pieces[INDEX_ROOK] == 0 &&
        black_pieces[INDEX_QUEEN] == 0)
    {
      for (i=0; i<8; i++) {
        for (j=0; j<8; j++) {
          piece = position->board[i][j];
          if (piece == ' ' || piece == 'k' || piece == 'K') continue;
          else if (piece == 'N') white_score += KNIGHT_VALUE;
          else if (piece == 'B') white_score += BISHOP_VALUE;
          else if (piece == 'R') white_score += ROOK_VALUE;
          else if (piece == 'Q') white_score += QUEEN_VALUE;
          else if (piece == 'P') {
            white_score += PAWN_VALUE;
            white_score += j*j * 5;
            continue;
          }
          white_score += 64 - abs(position->blackKingSquare_X - i) * 
            abs(position->blackKingSquare_Y - j);
        }
      }
      white_score += 160 - 10 *
        (abs(position->blackKingSquare_X - position->whiteKingSquare_X) + 
        abs(position->blackKingSquare_Y - position->whiteKingSquare_Y));
      i = position->blackKingSquare_X;
      j = position->blackKingSquare_Y;
      black_score += ((-i*i + 8*i) * (-j*j + 8*j));
      return white_score - black_score; 
    }
    // White has only a king
    if (white_pieces[INDEX_PAWN] == 0 && white_pieces[INDEX_KNIGHT] == 0 &&
        white_pieces[INDEX_BISHOP] == 0 && white_pieces[INDEX_ROOK] == 0 &&
        white_pieces[INDEX_QUEEN] == 0)
    {
      for (i=0; i<8; i++) {
        for (j=0; j<8; j++) {
          piece = position->board[i][j];
          if (piece == ' ' || piece == 'k' || piece == 'K') continue;
          else if (piece == 'n') black_score += KNIGHT_VALUE;
          else if (piece == 'b') black_score += BISHOP_VALUE;
          else if (piece == 'r') black_score += ROOK_VALUE;
          else if (piece == 'q') black_score += QUEEN_VALUE;
          else if (piece == 'p') {
            black_score += PAWN_VALUE;
            black_score += (7-j)*(7-j) * 5;
            continue;
          }
          black_score += 64 - abs(position->whiteKingSquare_X - i) * 
            abs(position->whiteKingSquare_Y - j);
        }
      }
      black_score += 160 - 10 *
        (abs(position->blackKingSquare_X - position->whiteKingSquare_X) + 
        abs(position->blackKingSquare_Y - position->whiteKingSquare_Y));
      i = position->whiteKingSquare_X;
      j = position->whiteKingSquare_Y;
      white_score += ((-i*i + 8*i) * (-j*j + 8*j));
      return white_score - black_score; 
    }    
  }

  // REAL EVALUATION
  
  for (i=0; i<8; i++) {
    for (j=0; j<8; j++) {
      piece = position->board[i][j];
      if (piece == ' ') continue;
      else if (piece == 'P') {
        was_passed_pawn = FALSE;
        white_score += pawnScore_w(position, i, j);
        if (j==6) white_score += PAWN_AT_SEVENTH;
        if (i==0) { 
          if (black_most_backward_pawn[0] <= j && 
              black_most_backward_pawn[1] <= j)
            was_passed_pawn = TRUE;
        }
        else if (i==7) { 
          if (black_most_backward_pawn[6] <= j && 
              black_most_backward_pawn[7] <= j)
            was_passed_pawn = TRUE;
        }
        else { 
          if (black_most_backward_pawn[i] <= j && 
              black_most_backward_pawn[i+1] <= j &&
              black_most_backward_pawn[i-1] <= j)
            was_passed_pawn = TRUE;
        }
        if (was_passed_pawn) {
          white_score += PASSED_PAWN;
          if (i==0) {
            if (position->board[i+1][j-1]=='P')
              white_score += PASSED_PAWN_PROTECTED;
          }
          else if (i==7) {
            if (position->board[i-1][j-1]=='P')
              white_score += PASSED_PAWN_PROTECTED;       
          }
          else {
            if (position->board[i+1][j-1]=='P' || 
                position->board[i-1][j-1]=='P')
              white_score += PASSED_PAWN_PROTECTED;
          }
          if (endgame) {
            if (position->whitesMove) {
              if (j>position->blackKingSquare_Y || 
                  (7-j)<abs(position->blackKingSquare_X-i))
                white_score += PASSED_PAWN_FAR_FROM_K;
            }
            else {
              if (j>position->blackKingSquare_Y+1 || 
                  (8-j)<abs(position->blackKingSquare_X-i))
                white_score += PASSED_PAWN_FAR_FROM_K;          
            }
          }
        }
      }
      else if (piece == 'p') { 
        was_passed_pawn = FALSE;
        black_score += pawnScore_b(position, i, j);
        if (j==1) black_score += PAWN_AT_SEVENTH;
        if (i==0) { 
          if (white_most_backward_pawn[0] >= j && 
              white_most_backward_pawn[1] >= j)
            was_passed_pawn = TRUE;
        }
        else if (i==7) { 
          if (white_most_backward_pawn[6] >= j && 
              white_most_backward_pawn[7] >= j)
            was_passed_pawn = TRUE;
        }
        else { 
          if (white_most_backward_pawn[i] >= j && 
              white_most_backward_pawn[i+1] >= j &&
              white_most_backward_pawn[i-1] >= j)
            was_passed_pawn = TRUE;
        }
        if (was_passed_pawn) {
          if (i==0) {
            if (position->board[i+1][j+1]=='p')
              black_score += PASSED_PAWN_PROTECTED;
          }
          else if (i==7) {
            if (position->board[i-1][j+1]=='p')
              black_score += PASSED_PAWN_PROTECTED;       
          }
          else {
            if (position->board[i+1][j+1]=='p' || 
                position->board[i-1][j+1]=='p')
              black_score += PASSED_PAWN_PROTECTED;
          }         
          black_score += PASSED_PAWN;
          if (endgame) {
            if (!position->whitesMove) {
              if (j<position->whiteKingSquare_Y || 
                  j<abs(position->whiteKingSquare_X-i))
                black_score += PASSED_PAWN_FAR_FROM_K;
            }
            else {
              if (j<position->whiteKingSquare_Y-1 || 
                  (j+1)<abs(position->whiteKingSquare_X-i))
                black_score += PASSED_PAWN_FAR_FROM_K;          
            }
          }
        }
      }
      else if (piece == 'N') { 
        white_score += knightScore_w(position, i, j); 
        if (i>0 && i<7 && j>3) {
          if (black_most_backward_pawn[i-1]<=j && 
              black_most_backward_pawn[i+1]<=j)
            if (position->board[i-1][j-1] == 'P' || 
                position->board[i+1][j-1] == 'P') 
              white_score += KNIGHT_AT_OUTPOST;
        }
      }
      else if (piece == 'n') {
        black_score += knightScore_b(position, i, j);
        if (i>0 && i<7 && j<4) {
          if (white_most_backward_pawn[i-1]>=j && 
              white_most_backward_pawn[i+1]>=j)
            if (position->board[i-1][j+1] == 'p' || 
                position->board[i+1][j+1] == 'p') 
              black_score += KNIGHT_AT_OUTPOST;
        }

      }
      else if (piece == 'B') {
        white_score += bishopScore_w(position, i, j);
        if (white_pieces[INDEX_BISHOP] == 1) {
          if ((i+j)%2)
            white_score += whitePawns_on_w * PAWN_ON_BISHOPS_COLOR;
          else white_score += whitePawns_on_b * PAWN_ON_BISHOPS_COLOR;
        }
      }
      else if (piece == 'b') {
        black_score += bishopScore_b(position, i, j);
        if (black_pieces[INDEX_BISHOP] == 1) {
          if ((i+j)%2)
            black_score += blackPawns_on_w * PAWN_ON_BISHOPS_COLOR;
          else black_score += blackPawns_on_b * PAWN_ON_BISHOPS_COLOR;
        }       
      }
      else if (piece == 'R') {
        white_score += rookScore_w(position, i, j);
        if (white_pawns_file[i] == 0) {
          if (i<3 || i>5) {
            if (abs(position->blackKingSquare_X - i) < 2)
              white_score += ROOK_THREATH;
          }
          if (black_pawns_file[i] == 0) white_score += ROOK_ON_EMPTY_FILE;
          else white_score += ROOK_ON_HALF_EMPTY_FILE;
        }
        if (j==6) white_score += ROOK_AT_SEVENTH;
      } 
      else if (piece == 'r') { 
        black_score += rookScore_b(position, i, j);
        if (black_pawns_file[i] == 0) {
          if (i<3 || i>5) {
            if (abs(position->whiteKingSquare_X - i) < 2)
              black_score += ROOK_THREATH;
          }
          if (white_pawns_file[i] == 0) black_score += ROOK_ON_EMPTY_FILE;
          else black_score += ROOK_ON_HALF_EMPTY_FILE;
        }
        if (j==1) black_score += ROOK_AT_SEVENTH;
      } 
      else if (piece == 'Q') {
        white_score += queenScore_w(position, i, j);
        if (state == endgame) {
          white_score += 200 - 20 * abs(position->blackKingSquare_X - i) +
            200 - 20 * abs(position->blackKingSquare_Y - j);
        }
        else {
          if (whiteDevelopment == 0)
            white_score += (-j*j + 8*j);
          else if (whiteDevelopment < -205 && j>2)
            white_score += QUEEN_TOO_EARLY;
        }
      }
      else if (piece == 'q') {
        black_score += queenScore_b(position, i, j);
        if (state == endgame) {
          black_score += 200 - 20 * abs(position->whiteKingSquare_X - i) +
            200 - 20 * abs(position->whiteKingSquare_Y - j);
        }
        else {
          if (blackDevelopment == 0)
            black_score += (-j*j + 6*j + 7);
          else if (blackDevelopment < -205 && j<6)
            black_score += QUEEN_TOO_EARLY;
        }
      } 
    }
  }

  // EVALUATE PAWN STRUCTURE

  // for white
  for (i=0; i<8; i++) {
    if (i == 0)
    {
      if (white_pawns_file[i] == 1) {
        if (white_pawns_file[1] == 0) white_score += ISOLATED_PAWN;
      }  
      else if (white_pawns_file[i] == 2) {
        if (white_pawns_file[1] == 0) white_score += ISOLATED_DOUBLED_PAWNS;
        else white_score += DOUBLED_PAWNS;
      }
      else if (white_pawns_file[i] > 2) {
        if (white_pawns_file[1] == 0) white_score += ISOLATED_TRIPLED_PAWNS;
        else white_score += TRIPLED_PAWNS;
      }
    }
    else if (i == 7)
    {
      if (white_pawns_file[i] == 1) {
        if (white_pawns_file[6] == 0) white_score += ISOLATED_PAWN;
      }  
      else if (white_pawns_file[i] == 2) {
        if (white_pawns_file[6] == 0) white_score += ISOLATED_DOUBLED_PAWNS;
        else white_score += DOUBLED_PAWNS;
      }
      else if (white_pawns_file[i] > 2) {
        if (white_pawns_file[6] == 0) white_score += ISOLATED_TRIPLED_PAWNS;
        else white_score += TRIPLED_PAWNS;
      }
    }
    else
    {
      if (white_pawns_file[i] == 1) {
        if (white_pawns_file[i+1] == 0 && white_pawns_file[i-1] == 0) 
          white_score += ISOLATED_PAWN;
      }  
      else if (white_pawns_file[i] == 2) {
        if (white_pawns_file[i+1] == 0 && white_pawns_file[i-1] == 0) 
          white_score += ISOLATED_DOUBLED_PAWNS;
        else white_score += DOUBLED_PAWNS;
      }
      else if (white_pawns_file[i] > 2) {
        if (white_pawns_file[i+1] == 0 && white_pawns_file[i-1] == 0) 
          white_score += ISOLATED_TRIPLED_PAWNS;
        else white_score += TRIPLED_PAWNS;
      }
    }
  }

  // for black
  for (i=0; i<8; i++) {
    if (i == 0)
    {
      if (black_pawns_file[i] == 1) {
        if (black_pawns_file[1] == 0) black_score += ISOLATED_PAWN;
      }  
      else if (black_pawns_file[i] == 2) {
        if (black_pawns_file[1] == 0) black_score += ISOLATED_DOUBLED_PAWNS;
        else black_score += DOUBLED_PAWNS;
      }
      else if (black_pawns_file[i] > 2) {
        if (black_pawns_file[1] == 0) black_score += ISOLATED_TRIPLED_PAWNS;
        else black_score += TRIPLED_PAWNS;
      }
    }
    else if (i == 7)
    {
      if (black_pawns_file[i] == 1) {
        if (black_pawns_file[6] == 0) black_score += ISOLATED_PAWN;
      }  
      else if (black_pawns_file[i] == 2) {
        if (black_pawns_file[6] == 0) black_score += ISOLATED_DOUBLED_PAWNS;
        else black_score += DOUBLED_PAWNS;
      }
      else if (black_pawns_file[i] > 2) {
        if (black_pawns_file[6] == 0) black_score += ISOLATED_TRIPLED_PAWNS;
        else black_score += TRIPLED_PAWNS;
      }
    }
    else
    {
      if (black_pawns_file[i] == 1) {
        if (black_pawns_file[i+1] == 0 && black_pawns_file[i-1] == 0) 
          black_score += ISOLATED_PAWN;
      }  
      else if (black_pawns_file[i] == 2) {
        if (black_pawns_file[i+1] == 0 && black_pawns_file[i-1] == 0) 
          black_score += ISOLATED_DOUBLED_PAWNS;
        else black_score += DOUBLED_PAWNS;
      }
      else if (black_pawns_file[i] > 2) {
        if (black_pawns_file[i+1] == 0 && black_pawns_file[i-1] == 0) 
          black_score += ISOLATED_TRIPLED_PAWNS;
        else black_score += TRIPLED_PAWNS;
      }
    }
  }

  if (state == normal)
  {
  
    // EVALUATE KING SAFETY
  
    // for white
    i = position->whiteKingSquare_X;
    j = position->whiteKingSquare_Y;
    if (position->whiteHasCastled && (i < 3 || i > 5) && j < 2) {
      white_score += KING_IN_CORNER;
      if (white_pawns_file[i] == 0) white_score += PAWN_MISSING_K;
      if (j==0 && i==6) {
        if (position->board[5][1] == 'P' && position->board[6][1] == 'P' &&
            position->board[7][1] == 'P') white_score += GOOD_PAWN_BARRIER;
        else if (position->board[5][1]=='P' && position->board[6][1]=='P' &&
                 position->board[7][2] == 'P') 
          white_score += OK_PAWN_BARRIER;
        else if (position->board[5][1]=='P' && position->board[6][2]=='P' &&
                 position->board[7][1] == 'P') 
          white_score += OK_PAWN_BARRIER;
      }
    }
    if (!position->whiteHasCastled && !position->whiteShortCastle &&
        !position->whiteLongCastle) white_score += KING_MOVED;
    white_score -= j*j*100; 
    
    // for black 
    i = position->blackKingSquare_X;
    j = position->blackKingSquare_Y;
    if (position->whiteHasCastled && (i < 3 || i > 5) && j > 5) {
      black_score += KING_IN_CORNER;
      if (black_pawns_file[i] == 0) black_score += PAWN_MISSING_K;
      if (j==7 && i==6) {
        if (position->board[5][6] == 'p' && position->board[6][6] == 'p' &&
            position->board[7][6] == 'p') black_score += GOOD_PAWN_BARRIER;
        else if (position->board[5][6]=='p' && position->board[6][6]=='p' &&
                 position->board[7][5] == 'p') 
          black_score += OK_PAWN_BARRIER;
        else if (position->board[5][6]=='p' && position->board[6][5]=='p' &&
                 position->board[7][6] == 'p') 
          black_score += OK_PAWN_BARRIER;
      }           
    }
    if (!position->blackHasCastled && !position->blackShortCastle &&
        !position->blackLongCastle) black_score += KING_MOVED;
    black_score -= (7-j)*(7-j)*100;
    
    // EVALUATE PIECES ON WRONG PLACES
    
    // for white
    if (position->board[3][1] == 'P')
      if (position->board[3][2] != ' ') white_score += CENTER_PAWN_BLOCKED;
    if (position->board[4][1] == 'P')
      if (position->board[4][2] != ' ') white_score += CENTER_PAWN_BLOCKED;
    
    // for black 
    if (position->board[3][6] == 'p')
      if (position->board[3][5] != ' ') black_score += CENTER_PAWN_BLOCKED;
    if (position->board[4][6] == 'p')
      if (position->board[4][5] != ' ') black_score += CENTER_PAWN_BLOCKED;
  
  }
  else // ENDGAME 
  {
    // white
    i = position->whiteKingSquare_X;
    j = position->whiteKingSquare_Y;
    white_score += (-i*i + 8*i) * (-j*j + 8*j);
    if (white_pieces[INDEX_BISHOP] == 2)
      white_score += BISHOP_PAIR;
    // black
    i = position->blackKingSquare_X;
    j = position->blackKingSquare_Y;
    black_score += (-i*i + 8*i) * (-j*j + 8*j);
    if (black_pieces[INDEX_BISHOP] == 2)
      black_score += BISHOP_PAIR;    
  }
  
  white_score += ((unsigned int)rand())%2;
  black_score += ((unsigned int)rand())%2;

  return white_score - black_score;
}


int evaluate(PGame game) 
{
  if (game->currentPosition.whitesMove)
    return evaluate_position(game);
  else return -evaluate_position(game);
}







