/******************************************************************************
 *
 *  Hoplite (c) Frans Ekman
 *
 *  Static evaluation of a position 
 *
 *  modified 24.2.2004
 *
 *****************************************************************************/

#ifndef EVALUATE_H
#define EVALUATE_H

#include "game.h"

#define INDEX_PAWN   0
#define INDEX_KNIGHT 1
#define INDEX_BISHOP 2
#define INDEX_ROOK   3
#define INDEX_QUEEN  4

#define PAWN_VALUE     1000
#define KNIGHT_VALUE   3000
#define BISHOP_VALUE   3150
#define ROOK_VALUE     5000
#define QUEEN_VALUE    9000

#define UNDEVELOPED_CENTERPAWN  -200
#define UNDEVELOPED_K_KNIGHT    -250
#define UNDEVELOPED_K_BISHOP    -200
#define UNDEVELOPED_Q_BISHOP    -150
#define UNDEVELOPED_Q_KNIGHT    -100

#define PAWN_ADV_1SQUARE          75
#define PAWN_ADV_2SQUARE         150
#define PASSED_PAWN              300
#define DOUBLED_PAWNS           -140
#define TRIPLED_PAWNS           -600
#define ISOLATED_PAWN           -100
#define ISOLATED_DOUBLED_PAWNS  -400
#define ISOLATED_TRIPLED_PAWNS  -800
#define CENTER_PAWN_BLOCKED     -600
#define QUEEN_TOO_EARLY         -200
#define ROOK_ON_HALF_EMPTY_FILE  250
#define ROOK_ON_EMPTY_FILE       300
#define KING_IN_CORNER           300
#define KING_MOVED              -500
#define PAWN_MISSING_K          -300
#define ROOK_THREATH             500
#define KNIGHT_AT_OUTPOST        400
#define ROOK_AT_SEVENTH          500
#define PAWN_AT_SEVENTH          500
#define PASSED_PAWN_FAR_FROM_K   500
#define PASSED_PAWN_PROTECTED    300
#define PAWN_ON_BISHOPS_COLOR    -50
#define BISHOP_PAIR              300

#define GOOD_PAWN_BARRIER        250
#define OK_PAWN_BARRIER          150

#define PLACE_W ((-x*x + 8*x) + (-y*y + 8*y))
#define PLACE_B ((-x*x + 8*x) + (-y*y + 6*y + 7))

typedef enum {normal, endgame} EState;

int evaluateWhiteDevelopment(PPosition position);
int evaluateBlackDevelopment(PPosition position);

int pawnScore_w(PPosition position, int x, int y);
int knightScore_w(PPosition position, int x, int y);
int bishopScore_w(PPosition position, int x, int y);
int rookScore_w(PPosition position, int x, int y);
int queenScore_w(PPosition position, int x, int y);

int pawnScore_b(PPosition position, int x, int y);
int knightScore_b(PPosition position, int x, int y);
int bishopScore_b(PPosition position, int x, int y); 
int rookScore_b(PPosition position, int x, int y);
int queenScore_b(PPosition position, int x, int y);

int evaluate_position(PGame game);

int evaluate(PGame game);

#endif
