/******************************************************************************
 *                                                                            *
 *  Hoplite (c) Frans Ekman 2003                                              *
 *                                                                            *
 *  I use hashtables with zobristkeys for the transpositiontables. An old     *
 *  value in the hashtable is written over when the depth of the search was   *
 *  greater or equal to the older. Because in an alpha-beta search we do not  *
 *  allways know exactly the value of a position, so I store that to a        *
 *  variable (flag) in the struct to. If the real value is greather, then     *
 *  flag = HASH_BETA, if it's lower, then flag = HASH_ALPHA. Those inexact    *
 *  values can sometimes be used. The best move stored can allways be used,   *
 *  because it is only used for move ordering.                                *
 *                                                                            *
 *  modified 24.2.2004                                                        *
 *                                                                            *
 *****************************************************************************/

#ifndef HASH_H
#define HASH_H

#include "position.h"

#define HASHTABLE_SIZE 200000

#define HASH_EXACT 0
#define HASH_ALPHA 1
#define HASH_BETA  2

#define HASH_UNKNOWN_VALUE 99999999


typedef struct {
  zobristkey key;
  short int depth;
  short int flags;
  int value;
  TMove bestMove;
} THashe;


typedef THashe *PHashe;


extern THashe hashTable[HASHTABLE_SIZE];


int probeHash(int depth, int alpha, int beta, zobristkey key, PMove move);

void recordHash(int depth, int value, int hashf, zobristkey key, PMove move);

void clearHash();


#endif


