/******************************************************************************
 *                                                                            *
 *  Hoplite (c) Frans Ekman                                                   *
 *                                                                            *
 *  The iterate function implementation.
 *                                                                            *
 *  modified 24.2.2004                                                        *
 *                                                                            *
 *****************************************************************************/

#include "iterate.h"

TMove iterate(PGame game, double minTime, double maxTime)
{
  int depth;
  int score = 0;
  int alpha = -BIG_SCORE;
  int beta = BIG_SCORE;

  TLine principalVariation;
  TMoveScoreList theMoveScores1ply;
  TMove bestMove;

  evaluations = 0;
  nodes = 0;
  timeOfStart = dtime();
  searchInterrupted = FALSE;
  bestMove.promotion = 'F';

  /* we do not know the scores for the moves at ply 1. */
  knowMoveScores = FALSE;

  /* Call search() repeatedly with a bigger depth value each time, starting
     from 1.*/
  for (depth = 1;; depth++) {

    /* Don't start a new search if we have searched longer than the minimum 
       time. */
    if (dtime() - timeOfStart > minTime)
      break;

    /* Search first with an aspiration window. If the search fails, search
       again without a window */
    while (!searchInterrupted) {
      playPVLine = TRUE;
      score = search(game, depth, alpha, beta, &principalVariation, 
                     maxTime, 0, (depth + 8), TRUE, FALSE);
      if (score <= alpha || score >= beta) {
        alpha = -BIG_SCORE;
        beta = BIG_SCORE;
        // put everything back like it was before the search;
        moveScoresAtFirstPly = theMoveScores1ply;
        continue;
      }
      break;
    }

    alpha = score - WINDOW_WIDTH;
    beta = score + WINDOW_WIDTH;

    /* If the search got interrupted because the time ran out, the information
       from the search is useless */
    if (!searchInterrupted) {
      qsort(moveScoresAtFirstPly.moveScore, moveScoresAtFirstPly.firstFree,
            sizeof(TMoveScore), cmpMoveScores);
      knowMoveScores = TRUE;
      bestLine = principalVariation;
      bestMove = bestLine.argmove[0];
      printPVLine(game, depth, score, &principalVariation); 
    }

    /* There is no need for a new search if we found a mate */
    if (score >= MUST_BE_MATE && !searchInterrupted)
      break;

    theMoveScores1ply = moveScoresAtFirstPly;
  }

  return bestMove;
}


void printPVLine(PGame game, int depth, int score, TLine *pv)
{
  int scoreToPrint, time;

  if (!(game->currentPosition.whitesMove))
    scoreToPrint = (int)(-score/10);
  else 
    scoreToPrint = (int)(score/10);

  time = (int)((dtime() - timeOfStart) * 100);

  printf("%d %d %d %d ", depth, scoreToPrint, time, nodes);
  printLine(pv);
  printf("\n");
  fflush(stdout);
}



