/******************************************************************************
 *                                                                            *
 *  Hoplite,                                                                  *
 *  copyright 2003 by Frans Ekman                                             *
 *                                                                            *
 *  version 2.1.1                                                             *
 *                                                                            *
 *                                                                            *
 *  Hoplite, Copyright Frans Ekman 2003. No part of this software may be      *
 *  reproduced in any form without written permission of the author, Frans    *
 *  Ekman. This program is a freeware product without warranty. No liability  *
 *  shall be assumed for direct or consequential damage caused by the use of  *
 *  this software. You can copy this software to friends, collegues, etc, as  *
 *  long as the package stays unchanged, complete, and the copies gives       *
 *  obvious credit to the author Frans Ekman.                                 *
 *                                                                            *
 *                                                                            *
 *  modified 15.3.2004                                                        *
 *                                                                            *
 *****************************************************************************/


#include <stdio.h>
#include <string.h>
#include "search.h"
#include "iterate.h"
#include "engine.h"
#include "openingbook.h"

int main(void) 
{
  char input[255];

#ifdef DEBUG
  int score;
  TGame game;
  PMove move;
  PMoveList list;
  zobristkey testKey, testKey2 = 0;
  int legal;
  int i;
#endif

#ifndef DEBUG
  signal(SIGINT, SIG_IGN);
  signal(SIGTERM, SIG_IGN);
#endif

  srand(time(NULL));

  scanf("%s", input);
  if (strcmp(input, "xboard")==0) {
    playInXboard();
  } else {
    printf("Hoplite 2.1.1\n");
    printf("(c) Frans Ekman\n");
#ifdef DEBUG
    printf("wait...");
    generateMask();
    generateZobristKeys();
    clearHash();
    initiallizeHistoryMoves();
    initiallize_openings();
    printf("OK\n");
    setStart(&game);
    while (1) {
      printf("__________________________________________________\n\n");
      printPosition(&(game.currentPosition));
      printf("#a - analyze\n");
      printf("#e - evaluate\n");
      printf("#z get stored Zobrist key\n");
      printf("#o get move from openingbook");
      printf("or just type a move\n");
      scanf("%s", input);
      if (input[0] == '#') {
        if (input[1] == 'a') {
          evaluations = 0;
          iterate(&game, 60, 60);
          clearHash();
          if (!(game.currentPosition.whitesMove))
            score = -score;
          //printf("analysed score = %d\n", score/10);
          //printf("evaluations: %d\n", evaluations);
          //printf("line: ");
          //printLine(&principalVariation);
          printf("\n");
        } else if (input[1] == 'o') {
          printf("-> ");
          move = getMoveFromOpenings(&game);
          p_move(move, stdout);
        } else if (input[1] == 'z') {
          testKey = getZobristKey(&(game.currentPosition.board),
                                  game.currentPosition.whitesMove);
          printf("zobristkey should be: %lx \n", testKey);
          printf("the real zobristkey for this position is: %lx\n ",
                 game.currentPosition.key);
          if (testKey != game.currentPosition.key)
            printf("KEYS NOT EQUAL!!!!\n");
          //printf("%d\n", game.currentPosition.allPieces);
        }  
      } else if (strncmp(input, "setboard", 8)==0) {
        getc(stdin);
        game.firstFree = 0;
        for (i=0; i<255; i++) {
          input[i] = getc(stdin);
          if (input[i] == '\0' || input[i] == '\n' || input[i] == EOF)
            break;
        }
        setPosition(&(game.currentPosition), input);
        game.currentPosition.knowIfCheck = FALSE;
        evaluateCheck(&(game.currentPosition));
      } else {
        if (!(isalpha(input[0]) && isdigit(input[1]) &&
              isalpha(input[2]) && isdigit(input[3]))) continue;
        move = createMove(&(game.currentPosition), input);
        list = generateMoves(&(game.currentPosition));
        legal = FALSE;
        for (i=0; i<list->firstFree; i++) { 
          if (move->x1 == list->moves[i]->x1 &&
              move->x2 == list->moves[i]->x2 &&
              move->y1 == list->moves[i]->y1 &&
              move->y2 == list->moves[i]->y2 &&
              move->promotion == list->moves[i]->promotion) {
            makeMove(&game, move);
            oneMoveMade();
            legal = TRUE;
            break;
          } 
        }
        if (!legal)
          printf("Illegal move!!!\n");
        free(move);
        freeMoveList(list);
      }
    }
#endif // DEBUG  
  }

  return 0;
}



