/******************************************************************************
 *                                                                            *
 *  Hoplite (c) Frans Ekman 2003                                              *
 *                                                                            *
 *  Position function implementations, see position.h.                        *
 *                                                                            *
 *  modified 7.3.2004                                                         *
 *                                                                            *
 *****************************************************************************/


#include "position.h"

TBitboard mask[64];
TBitboard wq_castle_mask;
TBitboard wk_castle_mask;
TBitboard bq_castle_mask;
TBitboard bk_castle_mask;
TBitboard wq_castle;
TBitboard wk_castle;
TBitboard bq_castle;
TBitboard bk_castle;

void generateMask(void)
{
  // normal mask
  int i;
  for (i=0; i<64; i++)
    mask[i] = (TBitboard) 1 << i;
  // castle masks
  wq_castle_mask = mask[7] | mask[6] | mask[5] | mask[4] | mask[3];
  wq_castle_mask = ~wq_castle_mask;
  wk_castle_mask = mask[0] | mask[1] | mask[2] | mask[3];
  wk_castle_mask = ~wk_castle_mask;
  bq_castle_mask = mask[63] | mask[62] | mask[61] | mask[60] | mask[59];
  bq_castle_mask = ~bq_castle_mask;
  bk_castle_mask = mask[56] | mask[57] | mask[58] | mask[59];
  bk_castle_mask = ~bk_castle_mask;
  // castle bitboards
  wq_castle = mask[4] | mask[5];
  wk_castle = mask[1] | mask[2];
  bq_castle = mask[60] | mask[61];
  bk_castle = mask[57] | mask[58];
}


void freeMoveList(PMoveList list)
{
  int i=0;
  while (i < list->firstFree) { free(list->moves[i]); i++; }
  free(list);
}


PMove newMove(short int x1, short int y1, short int x2, short int y2, 
              char promotion, short int capiture, char piece)
{
  PMove move = (PMove)malloc(sizeof(*move));
  move->x1 = x1;
  move->y1 = y1;
  move->x2 = x2;
  move->y2 = y2;
  move->promotion = promotion;
  move->capiture = capiture;
  move->piece = piece;
  return move;
}


PMove createMove(PPosition position, char *moveString)
{
  PMove move = (PMove)malloc(sizeof(*move));
  move->x1 = convertX(moveString[0]);
  move->y1 = atoi(moveString+1) - 1;
  move->x2 = convertX(moveString[2]);
  move->y2 = atoi(moveString+3)-1;
  if (moveString[4] != '\0')
    move->promotion = moveString[4];
  else move->promotion = NO_PROMOTION;
  move->piece = tolower(position->board[move->x1][move->y1]);
  if (position->board[move->x2][move->y2] != ' ')
    move->capiture = TRUE;
  else if (move->piece == 'p' && position->enPeassant == move->x2)
    if ((position->whitesMove && move->y2 == 5) ||
        (!position->whitesMove && move->y2 == 2))
      move->capiture = TRUE;
    else move->capiture = FALSE;
  else move->capiture = FALSE;
  return move;  
}


void add(PMoveList list, PMove move, PPosition position) 
{
  if (isLegal(position, move)) {
    list->moves[list->firstFree] = move;
    list->firstFree++;
  }
  else free(move);
}


void addWithoutCheck(PMoveList list, PMove move)
{
  list->moves[list->firstFree] = move;
  list->firstFree++;
}


int toSquareNumber(int x, int y) { return 7-x + 8*y; }

int getX(int square) { return 7 - square%8; }

int getY(int square) { return square / 8; }

int convertX(char letter)
{
  switch (letter)
  {
  case 'a' : return 0;
  case 'b' : return 1;
  case 'c' : return 2;
  case 'd' : return 3;
  case 'e' : return 4;
  case 'f' : return 5;
  case 'g' : return 6;
  case 'h' : return 7;
  default  : return NO_ENPEASSANT;
  }
  return NO_ENPEASSANT;
}   


int convertToColumn(int x)
{
  switch (x)
  {
  case 0 : return 'a';
  case 1 : return 'b';
  case 2 : return 'c';
  case 3 : return 'd';
  case 4 : return 'e';
  case 5 : return 'f';
  case 6 : return 'g';
  case 7 : return 'h';
  default  : return ' ';
  }
}


int setPosition(PPosition position, char *fenString)
{
  int x = 0, y = 7;
  int i=0, j, temp;
  char c;
  //position->allPieces = 0;
  position->n_white_pieces = 0;
  position->n_black_pieces = 0;
  position->movesSince = 0;
  // Put the pieces on the board
  while (y>=0 && x<=7) {
    c = *(fenString+i);
    if (isdigit(c)) {
      temp = atoi(fenString + i);
      for (j=0; j<temp; j++) 
        position->board[x+j][y] = ' '; 
      x += temp-1;
    }
    else if (c == '/') {
      y--;
      x = -1;
    }
    else {
      if (c == 'K') {
        position->whiteKingSquare_X = x;
        position->whiteKingSquare_Y = y;
      }
      else if (c == 'k') {
        position->blackKingSquare_X = x;
        position->blackKingSquare_Y = y;
      }
      position->board[x][y] = c;
      if (isupper(c)) 
        position->n_white_pieces++;
      else
        position->n_black_pieces++;
      //position->allPieces |= mask[toSquareNumber(x, y)];
    }
    i++;
    x++; 
    if (x > 7 && y>0) x=7;
  }
  i++; 
  // Check if it is whites turn to move
  c = *(fenString + i);
  if (c == 'w') position->whitesMove = TRUE;
  else position->whitesMove = FALSE;

  // Zobrist key
  position->key = getZobristKey(&(position->board), position->whitesMove);

  i += 2;
  position->whiteShortCastle = FALSE; 
  position->blackShortCastle = FALSE; 
  position->whiteLongCastle  = FALSE; 
  position->blackLongCastle  = FALSE;
  c = *(fenString + i);
  if (c != '-') { 
    // White kingside castle
    if (c == 'K') {
      position->whiteShortCastle = TRUE;
      i++;
    }
    // White queenside castle
    c = *(fenString + i);
    if (c == 'Q') {
      position->whiteLongCastle = TRUE;
      i++;
    }
    // Black kingside castle
    c = *(fenString + i);
    if (c == 'k') {
      position->blackShortCastle = TRUE;
      i++;
    }
    // Black queenside castle
    c = *(fenString + i);
    if (c == 'q') {
      position->blackLongCastle = TRUE;
      i++;
    }
  } else i++;
  i++;
  // Check en-peassant
  c = *(fenString + i);
  if (c == '-') position->enPeassant = NO_ENPEASSANT;
  else { 
    position->enPeassant = convertX(c);
    i++;
  }
  i+=2;
  // check half moves since last pawnmove or capiture
  position->movesSince = atoi(fenString + i);
  position->knowIfCheck = FALSE;
  if (position->whiteKingSquare_X == 4 && position->whiteKingSquare_Y == 0)
    position->whiteHasCastled = FALSE;
  else position->whiteHasCastled = TRUE;
  if (position->blackKingSquare_X == 4 && position->blackKingSquare_Y == 7)
    position->blackHasCastled = FALSE;
  else position->blackHasCastled = TRUE;

  return 0;
}


void setStartPosition(PPosition position)
{
  setPosition(position, 
              "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1\0");
  position->knowIfCheck = TRUE;
  position->isCheck = FALSE;
}


void removeCastleRights(PPosition position)
{
  if (position->whitesMove) {
    position->whiteShortCastle = FALSE;
    position->whiteLongCastle  = FALSE; 
  }
  else { 
    position->blackShortCastle = FALSE;
    position->blackLongCastle  = FALSE;
  }
}


void doMove(PPosition position, PMove move)
{
  position->knowIfCheck = FALSE;
  position->movesSince++;
  if (!move->capiture) {
    // king move
    if (move->piece == 'k') {
      // if it's a castle move
      if (abs(move->x1 - move->x2) > 1) {
        if (position->whitesMove) { 
          // White Queenside Castle
          if (move->x2 == 2) { 
            position->board[0][0] = ' ';
            position->board[2][0] = 'K';
            position->board[3][0] = 'R';
            position->board[4][0] = ' ';
            position->whiteKingSquare_X = 2;
            //position->allPieces &= wq_castle_mask;
            //position->allPieces |= wq_castle;
            removeCastleRights(position);
            position->whiteHasCastled = TRUE;
            // update Zobrist key
            zobrist_wq_castle(&(position->key));
          }
          // White Kingside Castle
          else {
            position->board[4][0] = ' ';
            position->board[5][0] = 'R';
            position->board[6][0] = 'K';
            position->board[7][0] = ' ';
            position->whiteKingSquare_X = 6;
            //position->allPieces &= wk_castle_mask;
            //position->allPieces |= wk_castle;
            removeCastleRights(position);
            position->whiteHasCastled = TRUE;
            // update Zobrist key
            zobrist_wk_castle(&(position->key));
          }
        }
        else {
          // Black Queenside Castle
          if (move->x2 == 2) { 
            position->board[0][7] = ' ';
            position->board[2][7] = 'k';
            position->board[3][7] = 'r';
            position->board[4][7] = ' ';
            position->blackKingSquare_X = 2;
            //position->allPieces &= bq_castle_mask;
            //position->allPieces |= bq_castle;
            removeCastleRights(position);
            position->blackHasCastled = TRUE;
            // update Zobrist key
            zobrist_bq_castle(&(position->key));
          }
          // Black Kingside Castle
          else {
            position->board[4][7] = ' ';
            position->board[5][7] = 'r';
            position->board[6][7] = 'k';
            position->board[7][7] = ' ';
            position->blackKingSquare_X = 6;
            //position->allPieces &= bk_castle_mask;
            //position->allPieces |= bk_castle;
            removeCastleRights(position);
            position->blackHasCastled = TRUE;
            // update Zobrist key
            zobrist_bk_castle(&(position->key));
          }
        }
        position->whitesMove = ! position->whitesMove;
        position->enPeassant = NO_ENPEASSANT;
        return;
      }
      // normal king move
      else {
        position->board[move->x2][move->y2] = 
          position->board[move->x1][move->y1];
        position->board[move->x1][move->y1] = ' ';
        //position->allPieces &= ~mask[toSquareNumber(move->x1, move->y1)];
        //position->allPieces |= mask[toSquareNumber(move->x2, move->y2)];
        if (position->whitesMove) {
          position->whiteKingSquare_X = move->x2; 
          position->whiteKingSquare_Y = move->y2;
          // update zobrist key
          addPiece(&(position->key), 'K', move->x1, move->y1);
          addPiece(&(position->key), 'K', move->x2, move->y2);
          switchTurnToMove(&(position->key));
        }
        else {
          position->blackKingSquare_X = move->x2; 
          position->blackKingSquare_Y = move->y2;
          // update zobrist key
          addPiece(&(position->key), 'k', move->x1, move->y1);
          addPiece(&(position->key), 'k', move->x2, move->y2);
          switchTurnToMove(&(position->key));
        }
        position->enPeassant = NO_ENPEASSANT;
        removeCastleRights(position);
        position->whitesMove = ! position->whitesMove;
        return;
      }
    }
    // not kingmove
    else {
      if (move->promotion != NO_PROMOTION) {
        if (position->whitesMove) {
          position->board[move->x2][move->y2] = toupper(move->promotion);
          // update zobrist key
          addPiece(&(position->key), 'P', move->x1, move->y1);
          addPiece(&(position->key), toupper(move->promotion), 
                   move->x2, move->y2);
          switchTurnToMove(&(position->key));
        }
        else {
          position->board[move->x2][move->y2] = move->promotion;
          // update zobrist key
          addPiece(&(position->key), 'p', move->x1, move->y1);
          addPiece(&(position->key), tolower(move->promotion), 
                   move->x2, move->y2);
          switchTurnToMove(&(position->key));
        }
      }
      else {
        position->board[move->x2][move->y2] = 
          position->board[move->x1][move->y1];;
        // update zobrist key
        addPiece(&(position->key), position->board[move->x2][move->y2],
                 move->x1, move->y1);
        addPiece(&(position->key), position->board[move->x2][move->y2],
                 move->x2, move->y2);
        switchTurnToMove(&(position->key));
      }
      position->board[move->x1][move->y1] = ' ';
      //position->allPieces &= ~mask[toSquareNumber(move->x1, move->y1)];
      //position->allPieces |= mask[toSquareNumber(move->x2, move->y2)];
      position->whitesMove = ! position->whitesMove;
      position->enPeassant = NO_ENPEASSANT;
      // if it was a pawnmove
      if (move->piece == 'p') {
        position->movesSince = 0;
        // if it moved 2 squares
        if (abs(move->y2 - move->y1) > 1)
          position->enPeassant = move->x1;
        return;
      }
      if (move->piece == 'r') {
        if (move->x1 == 0 && move->y1 == 0) 
          position->whiteLongCastle = FALSE;
        else if (move->x1 == 0 && move->y1 == 7) 
          position->blackLongCastle = FALSE;
        else if (move->x1 == 7 && move->y1 == 0) 
          position->whiteShortCastle = FALSE;
        else if (move->x1 == 7 && move->y1 == 7) 
          position->blackShortCastle = FALSE;
        else { }
      }
      return;
    }
  }
  // If the move is a capiture
  else {
    position->movesSince = 0;
    if (position->whitesMove)
      position->n_black_pieces--;
    else
      position->n_white_pieces--;

    // en peassant
    if (move->piece == 'p') { 
      if (position->whitesMove) { 
        if (move->y2 == 5 && move->x2 == position->enPeassant) {
          addPiece(&(position->key), position->board[move->x2][move->y2-1],
                   move->x2, move->y2-1);
          position->board[move->x2][move->y2-1] = ' ';
          //position->allPieces &= ~mask[toSquareNumber(move->x2, move->y2-1)];
        }
      }
      // blacks move 
      else {
        if (move->y2 == 2 && move->x2 == position->enPeassant) {
          addPiece(&(position->key), position->board[move->x2][move->y2+1],
                   move->x2, move->y2+1);
          position->board[move->x2][move->y2+1] = ' ';
          //position->allPieces &= ~mask[toSquareNumber(move->x2, move->y2+1)];
        }
      }
    }
    // Zobrist update for non en peassant capture move
    if (!(move->piece == 'p' && 
          ((move->y2 == 5 && position->whitesMove) || 
           (move->y2 == 2 && !(position->whitesMove))) && 
           move->x2 == position->enPeassant)) {
      addPiece(&(position->key), position->board[move->x2][move->y2],
               move->x2, move->y2);  
    }

    // update zobrist key
    addPiece(&(position->key), position->board[move->x1][move->y1],
             move->x1, move->y1);
    addPiece(&(position->key), position->board[move->x1][move->y1],
             move->x2, move->y2);
    switchTurnToMove(&(position->key));

    if (move->piece == 'r') {
      if (move->x1 == 0 && move->y1 == 0) 
        position->whiteLongCastle = FALSE;
      else if (move->x1 == 0 && move->y1 == 7) 
        position->blackLongCastle = FALSE;
      else if (move->x1 == 7 && move->y1 == 0) 
        position->whiteShortCastle = FALSE;
      else if (move->x1 == 7 && move->y1 == 7) 
        position->blackShortCastle = FALSE;
      else { }
    }
    if (move->x2 == 0 && move->y2 == 0) 
      position->whiteLongCastle = FALSE;
    else if (move->x2 == 0 && move->y2 == 7) 
      position->blackLongCastle = FALSE;
    else if (move->x2 == 7 && move->y2 == 0) 
      position->whiteShortCastle = FALSE;
    else if (move->x2 == 7 && move->y2 == 7) 
      position->blackShortCastle = FALSE;
    else { }
    position->enPeassant = NO_ENPEASSANT;
    if (move->piece == 'k') {
      if (position->whitesMove) {
        position->whiteKingSquare_X = move->x2; 
        position->whiteKingSquare_Y = move->y2;
      }
      else {
        position->blackKingSquare_X = move->x2; 
        position->blackKingSquare_Y = move->y2;
      }
      removeCastleRights(position);
    }
    if (move->promotion != NO_PROMOTION) {
      if (position->whitesMove) {
        position->board[move->x2][move->y2] = toupper(move->promotion);
        addPiece(&(position->key), 'P', move->x2, move->y2);
        addPiece(&(position->key), toupper(move->promotion), 
                 move->x2, move->y2);
      }
      else {
        position->board[move->x2][move->y2] = move->promotion;
        addPiece(&(position->key), 'p', move->x2, move->y2);
        addPiece(&(position->key), tolower(move->promotion), 
                 move->x2, move->y2);
      }
    }
    else
      position->board[move->x2][move->y2] = 
        position->board[move->x1][move->y1];
    position->board[move->x1][move->y1] = ' ';
    //position->allPieces &= ~mask[toSquareNumber(move->x1, move->y1)];
    //position->allPieces |= mask[toSquareNumber(move->x2, move->y2)];
    position->whitesMove = !position->whitesMove;
  }
}


int threath_lu_w(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    x2--;
    y2++;
    if (x2<0 || y2>7) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'b') return TRUE;
    else if (position->board[x2][y2] == 'q') return TRUE;
    else return FALSE;
  }
}


int threath_ru_w(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    x2++;
    y2++;
    if (x2>7 || y2>7) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'b') return TRUE;
    else if (position->board[x2][y2] == 'q') return TRUE;
    else return FALSE;
  }
}


int threath_ld_w(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    x2--;
    y2--;
    if (x2<0 || y2<0) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'b') return TRUE;
    else if (position->board[x2][y2] == 'q') return TRUE;
    else return FALSE;
  }
}


int threath_rd_w(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    x2++;
    y2--;
    if (x2>7 || y2<0) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'b') return TRUE;
    else if (position->board[x2][y2] == 'q') return TRUE;
    else return FALSE;
  }
}


int threath_u_w(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    y2++;
    if (y2>7) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'r') return TRUE;
    else if (position->board[x2][y2] == 'q') return TRUE;
    else return FALSE;
  }
}


int threath_d_w(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    y2--;
    if (y2<0) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'r') return TRUE;
    else if (position->board[x2][y2] == 'q') return TRUE;
    else return FALSE;
  }
}


int threath_l_w(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    x2--;
    if (x2<0) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'r') return TRUE;
    else if (position->board[x2][y2] == 'q') return TRUE;
    else return FALSE;
  }
}


int threath_r_w(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    x2++;
    if (x2>7) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'r') return TRUE;
    else if (position->board[x2][y2] == 'q') return TRUE;
    else return FALSE;
  }
}


int threath_other_w(PPosition position, int x, int y)
{
  // Pawns
  if (y<7) {
    if (x<7)
      if (position->board[x+1][y+1] == 'p') return TRUE;
    if (x>0)  
      if (position->board[x-1][y+1] == 'p') return TRUE;
  }
  // King
  if (abs(position->blackKingSquare_X - x) < 2 &&
      abs(position->blackKingSquare_Y - y) < 2) return TRUE;
  // Knights
  if (y<6) {
    if (x<7)
      if (position->board[x+1][y+2] == 'n') return TRUE;
    if (x>0)
      if (position->board[x-1][y+2] == 'n') return TRUE;
  }
  if (y<7) {
    if (x<6)
      if (position->board[x+2][y+1] == 'n') return TRUE;
    if (x>1)
      if (position->board[x-2][y+1] == 'n') return TRUE;
  }
  if (y>0) {
    if (x<6)
      if (position->board[x+2][y-1] == 'n') return TRUE;
    if (x>1)
      if (position->board[x-2][y-1] == 'n') return TRUE;    
  }
  if (y>1) {
    if (x<7)
      if (position->board[x+1][y-2] == 'n') return TRUE;
    if (x>0)
      if (position->board[x-1][y-2] == 'n') return TRUE;
  }
  return FALSE;
}





///////////////////////////////////////
// And for Black the same !!!!!!!!!!!


int threath_lu_b(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    x2--;
    y2++;
    if (x2<0 || y2>7) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'B') return TRUE;
    else if (position->board[x2][y2] == 'Q') return TRUE;
    else return FALSE;
  }
}


int threath_ru_b(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    x2++;
    y2++;
    if (x2>7 || y2>7) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'B') return TRUE;
    else if (position->board[x2][y2] == 'Q') return TRUE;
    else return FALSE;
  }
}


int threath_ld_b(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    x2--;
    y2--;
    if (x2<0 || y2<0) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'B') return TRUE;
    else if (position->board[x2][y2] == 'Q') return TRUE;
    else return FALSE;
  }
}


int threath_rd_b(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    x2++;
    y2--;
    if (x2>7 || y2<0) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'B') return TRUE;
    else if (position->board[x2][y2] == 'Q') return TRUE;
    else return FALSE;
  }
}


int threath_u_b(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    y2++;
    if (y2>7) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'R') return TRUE;
    else if (position->board[x2][y2] == 'Q') return TRUE;
    else return FALSE;
  }
}


int threath_d_b(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    y2--;
    if (y2<0) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'R') return TRUE;
    else if (position->board[x2][y2] == 'Q') return TRUE;
    else return FALSE;
  }
}


int threath_l_b(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    x2--;
    if (x2<0) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'R') return TRUE;
    else if (position->board[x2][y2] == 'Q') return TRUE;
    else return FALSE;
  }
}


int threath_r_b(PPosition position, int x, int y)
{
  int x2 = x;
  int y2 = y;
  while (1) {
    x2++;
    if (x2>7) return FALSE;
    else if (position->board[x2][y2] == ' ') continue;
    else if (position->board[x2][y2] == 'R') return TRUE;
    else if (position->board[x2][y2] == 'Q') return TRUE;
    else return FALSE;
  }
}


int threath_other_b(PPosition position, int x, int y)
{
  // Pawns
  if (y>0) {
    if (x<7)
      if (position->board[x+1][y-1] == 'P') return TRUE;
    if (x>0)  
      if (position->board[x-1][y-1] == 'P') return TRUE;
  }
  // King
  if (abs(position->whiteKingSquare_X - x) < 2 &&
      abs(position->whiteKingSquare_Y - y) < 2) return TRUE;
  // Knights
  if (y<6) {
    if (x<7)
      if (position->board[x+1][y+2] == 'N') return TRUE;
    if (x>0)
      if (position->board[x-1][y+2] == 'N') return TRUE;
  }
  if (y<7) {
    if (x<6)
      if (position->board[x+2][y+1] == 'N') return TRUE;
    if (x>1)
      if (position->board[x-2][y+1] == 'N') return TRUE;
  }
  if (y>0) {
    if (x<6)
      if (position->board[x+2][y-1] == 'N') return TRUE;
    if (x>1)
      if (position->board[x-2][y-1] == 'N') return TRUE;    
  }
  if (y>1) {
    if (x<7)
      if (position->board[x+1][y-2] == 'N') return TRUE;
    if (x>0)
      if (position->board[x-1][y-2] == 'N') return TRUE;
  }
  return FALSE;
}


int isSquareAttackedByBlack(PPosition position, int x, int y)
{
  if (threath_other_w(position, x, y)) return TRUE;
  if (threath_lu_w(position, x, y)) return TRUE;
  if (threath_ru_w(position, x, y)) return TRUE;
  if (threath_ld_w(position, x, y)) return TRUE;
  if (threath_rd_w(position, x, y)) return TRUE;
  if (threath_u_w(position, x, y)) return TRUE;
  if (threath_d_w(position, x, y)) return TRUE;
  if (threath_l_w(position, x, y)) return TRUE;
  if (threath_r_w(position, x, y)) return TRUE;
  return FALSE;
}


int isSquareAttackedByWhite(PPosition position, int x, int y)
{
  if (threath_other_b(position, x, y)) return TRUE;
  if (threath_lu_b(position, x, y)) return TRUE;
  if (threath_ru_b(position, x, y)) return TRUE;
  if (threath_ld_b(position, x, y)) return TRUE;
  if (threath_rd_b(position, x, y)) return TRUE;
  if (threath_u_b(position, x, y)) return TRUE;
  if (threath_d_b(position, x, y)) return TRUE;
  if (threath_l_b(position, x, y)) return TRUE;
  if (threath_r_b(position, x, y)) return TRUE;
  return FALSE;
}


void evaluateCheck(PPosition position)
{
  if (position->knowIfCheck) return;
  if (position->whitesMove)
    position->isCheck = isSquareAttackedByBlack(position, 
                                                position->whiteKingSquare_X,
                                                position->whiteKingSquare_Y);
  else 
    position->isCheck = isSquareAttackedByWhite(position, 
                                                position->blackKingSquare_X,
                                                position->blackKingSquare_Y);
  position->knowIfCheck = TRUE;
}


int resultInIllegalPositionForWhite(PPosition position, PMove move)
{
  PPosition temp;
  int dx = move->x1 - position->whiteKingSquare_X; 
  int dy = move->y1 - position->whiteKingSquare_Y;
  int adx = abs(dx);
  int ady = abs(dy);
  int illegal = FALSE;
  if (adx != ady && adx != 0 && ady != 0) return FALSE;
  temp = (PPosition)malloc(sizeof(TPosition));
  *temp = *position;
  doMove(temp, move);
  if (dx == 0) {
    if (dy > 0) // up
      illegal = threath_u_w(temp, temp->whiteKingSquare_X, 
                            temp->whiteKingSquare_Y);
    else // down
      illegal = threath_d_w(temp, temp->whiteKingSquare_X, 
                            temp->whiteKingSquare_Y);
  }
  else if (dy == 0) {
    if (dx > 0)  // right
      illegal = threath_r_w(temp, temp->whiteKingSquare_X, 
                            temp->whiteKingSquare_Y);
    else // left
      illegal = threath_l_w(temp, temp->whiteKingSquare_X, 
                            temp->whiteKingSquare_Y);
  }
  else if (dx > 0) {
    if (dy > 0) // right and up
      illegal = threath_ru_w(temp, temp->whiteKingSquare_X, 
                             temp->whiteKingSquare_Y);
    else // right and down
      illegal = threath_rd_w(temp, temp->whiteKingSquare_X, 
                             temp->whiteKingSquare_Y);
  }
  else {
    if (dy > 0) // left and up
      illegal = threath_lu_w(temp, temp->whiteKingSquare_X, 
                             temp->whiteKingSquare_Y);
    else // left and down
      illegal = threath_ld_w(temp, temp->whiteKingSquare_X, 
                             temp->whiteKingSquare_Y);  
  }
  free(temp);
  return illegal;
}



int resultInIllegalPositionForBlack(PPosition position, PMove move)
{
  PPosition temp;
  int dx = move->x1 - position->blackKingSquare_X; 
  int dy = move->y1 - position->blackKingSquare_Y;
  int adx = abs(dx);
  int ady = abs(dy);
  int illegal = FALSE;
  if (adx != ady && adx != 0 && ady != 0) return FALSE;
  temp = (PPosition)malloc(sizeof(TPosition));
  *temp = *position;
  doMove(temp, move);
  if (dx == 0) {
    if (dy > 0) // up
      illegal = threath_u_b(temp, temp->blackKingSquare_X, 
                            temp->blackKingSquare_Y);
    else // down
      illegal = threath_d_b(temp, temp->blackKingSquare_X, 
                            temp->blackKingSquare_Y);
  }
  else if (dy == 0) {
    if (dx > 0)  // right
      illegal = threath_r_b(temp, temp->blackKingSquare_X, 
                            temp->blackKingSquare_Y);
    else // left
      illegal = threath_l_b(temp, temp->blackKingSquare_X, 
                            temp->blackKingSquare_Y);
  }
  else if (dx > 0) {
    if (dy > 0) // right and up
      illegal = threath_ru_b(temp, temp->blackKingSquare_X, 
                             temp->blackKingSquare_Y);
    else // right and down
      illegal = threath_rd_b(temp, temp->blackKingSquare_X, 
                             temp->blackKingSquare_Y);
  }
  else {
    if (dy > 0) // left and up
      illegal = threath_lu_b(temp, temp->blackKingSquare_X, 
                             temp->blackKingSquare_Y);
    else // left and down
      illegal = threath_ld_b(temp, temp->blackKingSquare_X, 
                             temp->blackKingSquare_Y);  
  }
  free(temp);
  return illegal;
}


int isLegal(PPosition position, PMove move)
{
  PPosition temp;
  int legal = TRUE;
  if (position->whitesMove) {
    if (position->isCheck) {
      temp = (PPosition)malloc(sizeof(TPosition));
      *temp = *position;
      doMove(temp, move);
      legal = !isSquareAttackedByBlack(temp, temp->whiteKingSquare_X,
                                       temp->whiteKingSquare_Y);
      free(temp);
    }
    else {
      legal = !resultInIllegalPositionForWhite(position, move);
    }
  }
  else {
    if (position->isCheck) {
      temp = (PPosition)malloc(sizeof(TPosition));
      *temp = *position;
      doMove(temp, move);
      legal = !isSquareAttackedByWhite(temp, temp->blackKingSquare_X,
                                       temp->blackKingSquare_Y);
      free(temp);
    }
    else {
      legal = !resultInIllegalPositionForBlack(position, move);
    }
  }
  return legal;
}


void addPawnMoves(PPosition position, PMoveList list, int x, int y)
{
  if (position->whitesMove) {
    // one square forward empty
    if (position->board[x][y+1] == ' ') {
      if (y == 6) { // promotions
        add(list, newMove(x, y, x, y+1, 'n', 0, 'p'), position);
        add(list, newMove(x, y, x, y+1, 'b', 0, 'p'), position);
        add(list, newMove(x, y, x, y+1, 'r', 0, 'p'), position);
        add(list, newMove(x, y, x, y+1, 'q', 0, 'p'), position);
      }
      else { 
        add(list, newMove(x, y, x, y+1, NO_PROMOTION, 0, 'p'), position);
        // 2 square move
        if (y == 1 && position->board[x][y+2] == ' ')
          add(list, newMove(x, y, x, y+2, NO_PROMOTION, 0, 'p'), position);   
      }
    }
    // capitures
    if (x > 0) {
      // capiture left
      if (islower(position->board[x-1][y+1])) {
        if (y == 6) {
          add(list, newMove(x, y, x-1, y+1, 'n', 1, 'p'), position);
          add(list, newMove(x, y, x-1, y+1, 'b', 1, 'p'), position);
          add(list, newMove(x, y, x-1, y+1, 'r', 1, 'p'), position);
          add(list, newMove(x, y, x-1, y+1, 'q', 1, 'p'), position);
        }
        else add(list, newMove(x, y, x-1, y+1, NO_PROMOTION, 1, 'p'),position);
      }
    }
    if (x < 7) {
      // capiture right
      if (islower(position->board[x+1][y+1])) {
        if (y == 6) {
          add(list, newMove(x, y, x+1, y+1, 'n', 1, 'p'), position);
          add(list, newMove(x, y, x+1, y+1, 'b', 1, 'p'), position);
          add(list, newMove(x, y, x+1, y+1, 'r', 1, 'p'), position);
          add(list, newMove(x, y, x+1, y+1, 'q', 1, 'p'), position);
        }
        else add(list, newMove(x, y, x+1, y+1, NO_PROMOTION, 1, 'p'),position);
      }
    }
    // en peassant
    if (y == 4 && abs(position->enPeassant-x) == 1)
      add(list, newMove(x, 4, position->enPeassant, 5, NO_PROMOTION, 1, 'p'),
          position);
  }
  else { // Blacks move
// one square forward empty
    if (position->board[x][y-1] == ' ') {
      if (y == 1) { // promotions
        add(list, newMove(x, y, x, y-1, 'n', 0, 'p'), position);
        add(list, newMove(x, y, x, y-1, 'b', 0, 'p'), position);
        add(list, newMove(x, y, x, y-1, 'r', 0, 'p'), position);
        add(list, newMove(x, y, x, y-1, 'q', 0, 'p'), position);
      }
      else { 
        add(list, newMove(x, y, x, y-1, NO_PROMOTION, 0, 'p'), position);
        // 2 square move
        if (y == 6 && position->board[x][y-2] == ' ')
          add(list, newMove(x, y, x, y-2, NO_PROMOTION, 0, 'p'), position);   
      }
    }
    // capitures
    if (x > 0) {
      // capiture left
      if (isupper(position->board[x-1][y-1])) {
        if (y == 1) {
          add(list, newMove(x, y, x-1, y-1, 'n', 1, 'p'), position);
          add(list, newMove(x, y, x-1, y-1, 'b', 1, 'p'), position);
          add(list, newMove(x, y, x-1, y-1, 'r', 1, 'p'), position);
          add(list, newMove(x, y, x-1, y-1, 'q', 1, 'p'), position);
        }
        else add(list, newMove(x, y, x-1, y-1, NO_PROMOTION, 1, 'p'),position);
      }
    }
    if (x < 7) {
      // capiture right
      if (isupper(position->board[x+1][y-1])) {
        if (y == 1) {
          add(list, newMove(x, y, x+1, y-1, 'n', 1, 'p'), position);
          add(list, newMove(x, y, x+1, y-1, 'b', 1, 'p'), position);
          add(list, newMove(x, y, x+1, y-1, 'r', 1, 'p'), position);
          add(list, newMove(x, y, x+1, y-1, 'q', 1, 'p'), position);
        }
        else add(list, newMove(x, y, x+1, y-1, NO_PROMOTION, 1, 'p'),position);
      }
    }
    // en peassant
    if (y == 3 && abs(position->enPeassant-x) == 1)
      add(list, newMove(x, 3, position->enPeassant, 2, NO_PROMOTION, 1, 'p'), 
          position);
  }

}





void addKnightMoves(PPosition position, PMoveList list, int x, int y)
{
  if (position->whitesMove) {
    if (x>0 && y<6) {
      if (position->board[x-1][y+2] == ' ')
        add(list, newMove(x,y,(x-1),(y+2),NO_PROMOTION,0,'n'), position);
      else if (islower(position->board[x-1][y+2])) 
        add(list,newMove(x,y,(x-1),(y+2),NO_PROMOTION,1,'n'), position);
    }
    if (x<7 && y<6) {
      if (position->board[x+1][y+2] == ' ')
        add(list,newMove(x,y,(x+1),(y+2),NO_PROMOTION,0,'n'), position);
      else if (islower(position->board[x+1][y+2])) 
        add(list,newMove(x,y,(x+1),(y+2),NO_PROMOTION,1,'n'), position);
    }
    if (x<6 && y<7) {
      if (position->board[x+2][y+1] == ' ')
        add(list,newMove(x,y,(x+2),(y+1),NO_PROMOTION,0,'n'), position);
      else if (islower(position->board[x+2][y+1])) 
        add(list,newMove(x,y,(x+2),(y+1),NO_PROMOTION,1,'n'), position);
    }
    if (x<6 && y>0) {
      if (position->board[x+2][y-1] == ' ')
        add(list,newMove(x,y,(x+2),(y-1),NO_PROMOTION,0,'n'), position);
      else if (islower(position->board[x+2][y-1]))
        add(list,newMove(x,y,(x+2),(y-1),NO_PROMOTION,1,'n'), position);
    }
    if (x<7 && y>1) {
      if (position->board[x+1][y-2] == ' ')
        add(list,newMove(x,y,(x+1),(y-2),NO_PROMOTION,0,'n'), position);
      else if (islower(position->board[x+1][y-2]))
        add(list,newMove(x,y,(x+1),(y-2),NO_PROMOTION,1,'n'), position);
    }
    if (x>0 && y>1) {
      if (position->board[x-1][y-2] == ' ')
        add(list,newMove(x,y,(x-1),(y-2),NO_PROMOTION,0,'n'), position);
      else if (islower(position->board[x-1][y-2]))
        add(list,newMove(x,y,(x-1),(y-2),NO_PROMOTION,1,'n'), position);
    }
    if (x>1 && y>0) {
      if (position->board[x-2][y-1] == ' ')
        add(list,newMove(x,y,(x-2),(y-1),NO_PROMOTION,0,'n'), position);
      else if (islower(position->board[x-2][y-1]))
        add(list,newMove(x,y,(x-2),(y-1),NO_PROMOTION,1,'n'), position);
    }
    if (x>1 && y<7) {
      if (position->board[x-2][y+1] == ' ')
        add(list,newMove(x,y,(x-2),(y+1),NO_PROMOTION,0,'n'), position);
      else if (islower(position->board[x-2][y+1]))
        add(list,newMove(x,y,(x-2),(y+1),NO_PROMOTION,1,'n'), position);
    }
    return;
  }
  else { // BLACK
    if (x>0 && y<6) {
      if (position->board[x-1][y+2] == ' ')
        add(list,newMove(x,y,(x-1),(y+2),NO_PROMOTION,0,'n'), position);
      else if (isupper(position->board[x-1][y+2])) 
        add(list,newMove(x,y,(x-1),(y+2),NO_PROMOTION,1,'n'), position);
    }
    if (x<7 && y<6) {
      if (position->board[x+1][y+2] == ' ')
        add(list,newMove(x,y,(x+1),(y+2),NO_PROMOTION,0,'n'), position);
      else if (isupper(position->board[x+1][y+2])) 
        add(list,newMove(x,y,(x+1),(y+2),NO_PROMOTION,1,'n'), position);
    }
    if (x<6 && y<7) {
      if (position->board[x+2][y+1] == ' ')
        add(list,newMove(x,y,(x+2),(y+1),NO_PROMOTION,0,'n'), position);
      else if (isupper(position->board[x+2][y+1])) 
        add(list,newMove(x,y,(x+2),(y+1),NO_PROMOTION,1,'n'), position);
    }
    if (x<6 && y>0) {
      if (position->board[x+2][y-1] == ' ')
        add(list,newMove(x,y,(x+2),(y-1),NO_PROMOTION,0,'n'), position);
      else if (isupper(position->board[x+2][y-1]))
        add(list,newMove(x,y,(x+2),(y-1),NO_PROMOTION,1,'n'), position);
    }
    if (x<7 && y>1) {
      if (position->board[x+1][y-2] == ' ')
        add(list,newMove(x,y,(x+1),(y-2),NO_PROMOTION,0,'n'), position);
      else if (isupper(position->board[x+1][y-2]))
        add(list,newMove(x,y,(x+1),(y-2),NO_PROMOTION,1,'n'), position);
    }
    if (x>0 && y>1) {
      if (position->board[x-1][y-2] == ' ')
        add(list,newMove(x,y,(x-1),(y-2),NO_PROMOTION,0,'n'), position);
      else if (isupper(position->board[x-1][y-2]))
        add(list,newMove(x,y,(x-1),(y-2),NO_PROMOTION,1,'n'), position);
    }
    if (x>1 && y>0) {
      if (position->board[x-2][y-1] == ' ')
        add(list,newMove(x,y,(x-2),(y-1),NO_PROMOTION,0,'n'), position);
      else if (isupper(position->board[x-2][y-1]))
        add(list,newMove(x,y,(x-2),(y-1),NO_PROMOTION,1,'n'), position);
    }
    if (x>1 && y<7) {
      if (position->board[x-2][y+1] == ' ')
        add(list,newMove(x,y,(x-2),(y+1),NO_PROMOTION,0,'n'), position);
      else if (isupper(position->board[x-2][y+1]))
        add(list,newMove(x,y,(x-2),(y+1),NO_PROMOTION,1,'n'), position);
    }
    
  }
  
}


void addBishopMoves(PPosition position, PMoveList list, int x, int y, char c)
{
  int x2, y2;
  if (position->whitesMove) {
    x2=x;
    y2=y;
    // UP & LEFT
    while (1) {
      x2--;
      y2++;
      if (x2<0 || y2>7) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    y2=y;
    // UP & RIGHT
    while (1) {
      x2++;
      y2++;
      if (x2>7 || y2>7) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    y2=y;
    // DOWN & RIGHT
    while (1) {
      x2++;
      y2--;
      if (x2>7 || y2<0) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    y2=y;
    // DOWN & LEFT
    while (1) {
      x2--;
      y2--;
      if (x2<0 || y2<0) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
  }
  // FOR BLACK !!!!!!
  else {
    x2=x;
    y2=y;
    // UP & LEFT
    while (1) {
      x2--;
      y2++;
      if (x2<0 || y2>7) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    y2=y;
    // UP & RIGHT
    while (1) {
      x2++;
      y2++;
      if (x2>7 || y2>7) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    y2=y;
    // DOWN & RIGHT
    while (1) {
      x2++;
      y2--;
      if (x2>7 || y2<0) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    y2=y;
    // DOWN & LEFT
    while (1) {
      x2--;
      y2--;
      if (x2<0 || y2<0) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }

  }

}


void addRookMoves(PPosition position, PMoveList list, int x, int y, char c)
{
  int x2, y2;
  if (position->whitesMove) 
  {
    x2=x;
    y2=y;
    // UP
    while (1) {
      y2++;
      if (y2>7) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    y2=y;
    // RIGHT
    while (1) {
      x2++;
      if (x2>7) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    // DOWN
    while (1) {
      y2--;
      if (y2<0) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    y2=y;
    // LEFT
    while (1) {
      x2--;
      if (x2<0) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
  }
  else // SAME FOR BLACK !!!!!!
  { 
    x2=x;
    y2=y;
    // UP 
    while (1) {
      y2++;
      if (y2>7) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    y2=y;
    // RIGHT
    while (1) {
      x2++;
      if (x2>7) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    // DOWN
    while (1) {
      y2--;
      if (y2<0) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    y2=y;
    // LEFT
    while (1) {
      x2--;
      if (x2<0) break;
      if (position->board[x2][y2] == ' ') {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 0, c), position);
        continue;
      }
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }

  }

}


void addQueenMoves(PPosition position, PMoveList list, int x, int y)
{
  addBishopMoves(position, list, x, y, 'q');
  addRookMoves(position, list, x, y, 'q');
}


void addKingMoves(PPosition position, PMoveList list, int x, int y)
{
  int i, j;
  if (position->whitesMove) {
    for(i=-1; i<=1; i++) {
      for(j=-1; j<=1; j++) {
        if (x+i<0 || x+i>7 || y+j<0 || y+j>7)
          continue; 
        if (position->board[x+i][y+j] == ' ') {
          position->board[x][y] = ' ';
          if (!isSquareAttackedByBlack(position, x+i, y+j))
            addWithoutCheck(list,newMove(x,y,(x+i),(y+j),NO_PROMOTION,0,'k'));
          position->board[x][y] = 'K';
        }
        else if(islower(position->board[x+i][y+j])) {
          position->board[x][y] = ' ';
          if (!isSquareAttackedByBlack(position, x+i, y+j))
            addWithoutCheck(list,newMove(x,y,(x+i),(y+j),NO_PROMOTION,1,'k'));
          position->board[x][y] = 'K';
        }
      } 
    }
  }
  else {
    for(i=-1; i<=1; i++) {
      for(j=-1; j<=1; j++) {
        if (x+i<0 || x+i>7 || y+j<0 || y+j>7)
          continue; 
        if (position->board[x+i][y+j] == ' ') {
          position->board[x][y] = ' ';
          if (!isSquareAttackedByWhite(position, x+i, y+j))
            addWithoutCheck(list,newMove(x,y,(x+i),(y+j),NO_PROMOTION,0,'k'));
          position->board[x][y] = 'k';
        }
        else if(isupper(position->board[x+i][y+j])) {
          position->board[x][y] = ' ';
          if (!isSquareAttackedByWhite(position, x+i, y+j))
            addWithoutCheck(list,newMove(x,y,(x+i),(y+j),NO_PROMOTION,1,'k'));
          position->board[x][y] = 'k';
        } 
      } 
    }
  }
}


void addCastleMoves(PPosition position, PMoveList list)
{
  if (position->isCheck) return;
  if (position->whitesMove) {
    if (position->whiteShortCastle) {
      if (position->board[5][0] == ' ' && position->board[6][0] == ' ') 
        if (!isSquareAttackedByBlack(position, 5, 0))
          if (!isSquareAttackedByBlack(position, 6, 0)) {
            addWithoutCheck(list, newMove(4,0,6,0,NO_PROMOTION, 0, 'k'));
          }
    }
    if (position->whiteLongCastle) {
      if (position->board[3][0] == ' ' && position->board[2][0] == ' ' &&
          position->board[1][0] == ' ')
        if (!isSquareAttackedByBlack(position, 3, 0))
          if (!isSquareAttackedByBlack(position, 2, 0)) {
            addWithoutCheck(list, newMove(4,0,2,0,NO_PROMOTION, 0, 'k'));
          }
    }
  }
  else {
    if (position->blackShortCastle) {
      if (position->board[5][7] == ' ' && position->board[6][7] == ' ') 
        if (!isSquareAttackedByWhite(position, 5, 7))
          if (!isSquareAttackedByWhite(position, 6, 7)) {
            addWithoutCheck(list, newMove(4,7,6,7,NO_PROMOTION, 0, 'k'));
          }
    }
    if (position->blackLongCastle) {
      if (position->board[3][7] == ' ' && position->board[2][7] == ' ' &&
          position->board[1][7] == ' ')
        if (!isSquareAttackedByWhite(position, 3, 7))
          if (!isSquareAttackedByWhite(position, 2, 7)) {
            addWithoutCheck(list, newMove(4,7,2,7,NO_PROMOTION, 0, 'k'));
          }
    } 
  }
}


PMoveList generateMoves(PPosition position)
{
  int i, j;
  PMoveList list = (PMoveList)malloc(sizeof(TMoveList));
  list->firstFree = 0;
  addCastleMoves(position, list);
  // WHITE
  if (position->whitesMove) {
    for (i=0; i<8; i++)
      for (j=0; j<8; j++) {
        if (position->board[i][j] == ' ') continue;
        else if (position->board[i][j] == 'P')
          addPawnMoves(position, list, i, j);
        else if (position->board[i][j] == 'N')
          addKnightMoves(position, list, i, j);
        else if (position->board[i][j] == 'B')
          addBishopMoves(position, list, i, j, 'b');
        else if (position->board[i][j] == 'R')
          addRookMoves(position, list, i, j, 'r');
        else if (position->board[i][j] == 'Q')
          addQueenMoves(position, list, i, j);
        else if (position->board[i][j] == 'K')
          addKingMoves(position, list, i, j);
      }
  } 
  // BLACK
  else {
    for (i=0; i<8; i++)
      for (j=0; j<8; j++) {
        if (position->board[i][j] == ' ') continue;
        else if (position->board[i][j] == 'p')
          addPawnMoves(position, list, i, j);
        else if (position->board[i][j] == 'n')
          addKnightMoves(position, list, i, j);
        else if (position->board[i][j] == 'b')
          addBishopMoves(position, list, i, j, 'b');
        else if (position->board[i][j] == 'r')
          addRookMoves(position, list, i, j, 'r');
        else if (position->board[i][j] == 'q')
          addQueenMoves(position, list, i, j);
         else if (position->board[i][j] == 'k')
           addKingMoves(position, list, i, j);
      }
  }
  return list;
}


void printMove(PMove move)
{
  printf("%c%d%c%d", convertToColumn(move->x1), (move->y1 + 1),
         convertToColumn(move->x2), (move->y2 + 1));

  if (move->promotion != NO_PROMOTION)
    printf("%c", move->promotion);
  printf(", piece: %c, prom: %c ", move->piece, move->promotion);
  if (move->capiture) printf("capiture");

}


void p_move(PMove move, FILE *file) 
{
  fprintf(file, "%c%d%c%d", convertToColumn(move->x1), (move->y1 + 1),
         convertToColumn(move->x2), (move->y2 + 1));
  if (move->promotion != NO_PROMOTION)
    fprintf(file, "%c", move->promotion);
}

void printMoveList(PMoveList list)
{
  int i;
  for (i=0; i<list->firstFree; i++) {
    printMove(list->moves[i]);
    printf("\n");
  }
}


void printPosition(PPosition position)
{
  int x, y;
//printf("\n--------------------------------------------------------------\n");
  for (y=7; y>=0; y--) 
    for (x=0; x<8; x++) { 
      printf("%c", position->board[x][y]);
      if (x==7) printf("\n");
    }
  printf("\n");
/*
  printf("allPieces bitboard:\n");

  for (y=63; y>=0; y--) {
    if ((position->allPieces & mask[y]) == mask[y])
      printf("X");
    else printf("-");
    if (y%8==0) printf("\n");
  }
  if (position->whitesMove)
    printf("WHITES MOVE\n");
  else printf("BLACKS MOVE\n");
  printf("CASTLES: ");
  if (position->whiteLongCastle) printf("WQ ");
  if (position->whiteShortCastle) printf("WK ");
  if (position->blackLongCastle) printf("BQ ");
  if (position->blackShortCastle) printf("BK ");
  printf("\nMOVES SINCE PAWNMOVE OR CAPITURE: %d\n", position->movesSince);
  printf("WHITE KING: %d, %d\n", position->whiteKingSquare_X, 
         position->whiteKingSquare_Y);
  printf("BLACK KING: %d, %d\n", position->blackKingSquare_X, 
         position->blackKingSquare_Y);
  if (position->isCheck) printf("### CHECK ###\n");
*/
}


int equals(PPosition pos_a, PPosition pos_b)
{
  int i, j;
   
  if (pos_a->key == pos_b->key) {
    if (pos_a->whitesMove == pos_b->whitesMove) {
      for (i=0; i<8; i++)
        for (j=0; j<8; j++)
          if (pos_a->board[i][j] != pos_b->board[i][j])
            return FALSE;
      if (pos_a->whiteShortCastle != pos_b->whiteShortCastle) return FALSE;
      if (pos_a->blackShortCastle != pos_b->blackShortCastle) return FALSE;
      if (pos_a->whiteLongCastle != pos_b->whiteLongCastle) return FALSE;
      if (pos_a->blackLongCastle != pos_b->blackLongCastle) return FALSE;
      return TRUE;
    }
    else return FALSE;
  }
  else return FALSE;
}


int move_equals(PMove a, PMove b)
{
  if (a->x1 != b->x1) return FALSE;
  if (a->x2 != b->x2) return FALSE;
  if (a->y1 != b->y1) return FALSE;
  if (a->y2 != b->y2) return FALSE;
  if (a->piece != b->piece) return FALSE;
  if (a->promotion != b->promotion) return FALSE;
  return TRUE;
}


int insufficientMaterial(PPosition position) 
{
  int minor_pieces = 0;
  int i, j;
  char piece;
  for (i=0; i<8; i++) {
    for (j=0; j<8; j++) {
      piece = position->board[i][j];
      if (piece == 'p' || piece == 'P' || piece == 'Q' || piece == 'q')
        return FALSE;
      else if (piece == 'R' || piece == 'r') return FALSE;
      else if (piece == 'N' || piece == 'n' || piece == 'B' || piece == 'b')
        minor_pieces++;
      if (minor_pieces >= 2) return FALSE;
    }
  }
  return TRUE;
}






////////////////////////////////////////////////////
//                                                //
//  Capture move generation for quiesence search  //
//                                                //
////////////////////////////////////////////////////






void addPawnCaptures(PPosition position, PMoveList list, int x, int y)
{
  if (position->whitesMove) {
    // capitures
    if (x > 0) {
      // capiture left
      if (islower(position->board[x-1][y+1])) {
        if (y == 6) {
          add(list, newMove(x, y, x-1, y+1, 'n', 1, 'p'), position);
          add(list, newMove(x, y, x-1, y+1, 'b', 1, 'p'), position);
          add(list, newMove(x, y, x-1, y+1, 'r', 1, 'p'), position);
          add(list, newMove(x, y, x-1, y+1, 'q', 1, 'p'), position);
        }
        else add(list, newMove(x, y, x-1, y+1, NO_PROMOTION, 1, 'p'),position);
      }
    }
    if (x < 7) {
      // capiture right
      if (islower(position->board[x+1][y+1])) {
        if (y == 6) {
          add(list, newMove(x, y, x+1, y+1, 'n', 1, 'p'), position);
          add(list, newMove(x, y, x+1, y+1, 'b', 1, 'p'), position);
          add(list, newMove(x, y, x+1, y+1, 'r', 1, 'p'), position);
          add(list, newMove(x, y, x+1, y+1, 'q', 1, 'p'), position);
        }
        else add(list, newMove(x, y, x+1, y+1, NO_PROMOTION, 1, 'p'),position);
      }
    }
    // en peassant
    if (y == 4 && abs(position->enPeassant-x) == 1)
      add(list, newMove(x, 4, position->enPeassant, 5, NO_PROMOTION, 1, 'p'),
          position);
  }
  else { // Blacks move
    // capitures
    if (x > 0) {
      // capiture left
      if (isupper(position->board[x-1][y-1])) {
        if (y == 1) {
          add(list, newMove(x, y, x-1, y-1, 'n', 1, 'p'), position);
          add(list, newMove(x, y, x-1, y-1, 'b', 1, 'p'), position);
          add(list, newMove(x, y, x-1, y-1, 'r', 1, 'p'), position);
          add(list, newMove(x, y, x-1, y-1, 'q', 1, 'p'), position);
        }
        else add(list, newMove(x, y, x-1, y-1, NO_PROMOTION, 1, 'p'),position);
      }
    }
    if (x < 7) {
      // capiture right
      if (isupper(position->board[x+1][y-1])) {
        if (y == 1) {
          add(list, newMove(x, y, x+1, y-1, 'n', 1, 'p'), position);
          add(list, newMove(x, y, x+1, y-1, 'b', 1, 'p'), position);
          add(list, newMove(x, y, x+1, y-1, 'r', 1, 'p'), position);
          add(list, newMove(x, y, x+1, y-1, 'q', 1, 'p'), position);
        }
        else add(list, newMove(x, y, x+1, y-1, NO_PROMOTION, 1, 'p'),position);
      }
    }
    // en peassant
    if (y == 3 && abs(position->enPeassant-x) == 1)
      add(list, newMove(x, 3, position->enPeassant, 2, NO_PROMOTION, 1, 'p'), 
          position);
  }

}





void addKnightCaptures(PPosition position, PMoveList list, int x, int y)
{
  if (position->whitesMove) {
    if (x>0 && y<6) {
      if (islower(position->board[x-1][y+2])) 
        add(list,newMove(x,y,(x-1),(y+2),NO_PROMOTION,1,'n'), position);
    }
    if (x<7 && y<6) {
      if (islower(position->board[x+1][y+2])) 
        add(list,newMove(x,y,(x+1),(y+2),NO_PROMOTION,1,'n'), position);
    }
    if (x<6 && y<7) {
      if (islower(position->board[x+2][y+1])) 
        add(list,newMove(x,y,(x+2),(y+1),NO_PROMOTION,1,'n'), position);
    }
    if (x<6 && y>0) {
      if (islower(position->board[x+2][y-1]))
        add(list,newMove(x,y,(x+2),(y-1),NO_PROMOTION,1,'n'), position);
    }
    if (x<7 && y>1) {
      if (islower(position->board[x+1][y-2]))
        add(list,newMove(x,y,(x+1),(y-2),NO_PROMOTION,1,'n'), position);
    }
    if (x>0 && y>1) {
      if (islower(position->board[x-1][y-2]))
        add(list,newMove(x,y,(x-1),(y-2),NO_PROMOTION,1,'n'), position);
    }
    if (x>1 && y>0) {
      if (islower(position->board[x-2][y-1]))
        add(list,newMove(x,y,(x-2),(y-1),NO_PROMOTION,1,'n'), position);
    }
    if (x>1 && y<7) {
      if (islower(position->board[x-2][y+1]))
        add(list,newMove(x,y,(x-2),(y+1),NO_PROMOTION,1,'n'), position);
    }
    return;
  }
  else { // BLACK
    if (x>0 && y<6) {
      if (isupper(position->board[x-1][y+2])) 
        add(list,newMove(x,y,(x-1),(y+2),NO_PROMOTION,1,'n'), position);
    }
    if (x<7 && y<6) {
      if (isupper(position->board[x+1][y+2])) 
        add(list,newMove(x,y,(x+1),(y+2),NO_PROMOTION,1,'n'), position);
    }
    if (x<6 && y<7) {
      if (isupper(position->board[x+2][y+1])) 
        add(list,newMove(x,y,(x+2),(y+1),NO_PROMOTION,1,'n'), position);
    }
    if (x<6 && y>0) {
      if (isupper(position->board[x+2][y-1]))
        add(list,newMove(x,y,(x+2),(y-1),NO_PROMOTION,1,'n'), position);
    }
    if (x<7 && y>1) {
      if (isupper(position->board[x+1][y-2]))
        add(list,newMove(x,y,(x+1),(y-2),NO_PROMOTION,1,'n'), position);
    }
    if (x>0 && y>1) {
      if (isupper(position->board[x-1][y-2]))
        add(list,newMove(x,y,(x-1),(y-2),NO_PROMOTION,1,'n'), position);
    }
    if (x>1 && y>0) {
      if (isupper(position->board[x-2][y-1]))
        add(list,newMove(x,y,(x-2),(y-1),NO_PROMOTION,1,'n'), position);
    }
    if (x>1 && y<7) {
      if (isupper(position->board[x-2][y+1]))
        add(list,newMove(x,y,(x-2),(y+1),NO_PROMOTION,1,'n'), position);
    }
    
  }
  
}


void addBishopCaptures(PPosition position, PMoveList list, int x, 
                       int y, char c)
{
  int x2, y2;
  if (position->whitesMove) {
    x2=x;
    y2=y;
    // UP & LEFT
    while (1) {
      x2--;
      y2++;
      if (x2<0 || y2>7) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    y2=y;
    // UP & RIGHT
    while (1) {
      x2++;
      y2++;
      if (x2>7 || y2>7) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    y2=y;
    // DOWN & RIGHT
    while (1) {
      x2++;
      y2--;
      if (x2>7 || y2<0) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    y2=y;
    // DOWN & LEFT
    while (1) {
      x2--;
      y2--;
      if (x2<0 || y2<0) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
  }
  // FOR BLACK !!!!!!
  else {
    x2=x;
    y2=y;
    // UP & LEFT
    while (1) {
      x2--;
      y2++;
      if (x2<0 || y2>7) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    y2=y;
    // UP & RIGHT
    while (1) {
      x2++;
      y2++;
      if (x2>7 || y2>7) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    y2=y;
    // DOWN & RIGHT
    while (1) {
      x2++;
      y2--;
      if (x2>7 || y2<0) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    y2=y;
    // DOWN & LEFT
    while (1) {
      x2--;
      y2--;
      if (x2<0 || y2<0) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }

  }

}


void addRookCaptures(PPosition position, PMoveList list, int x, int y, char c)
{
  int x2, y2;
  if (position->whitesMove) 
  {
    x2=x;
    y2=y;
    // UP
    while (1) {
      y2++;
      if (y2>7) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    y2=y;
    // RIGHT
    while (1) {
      x2++;
      if (x2>7) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    // DOWN
    while (1) {
      y2--;
      if (y2<0) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    y2=y;
    // LEFT
    while (1) {
      x2--;
      if (x2<0) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (islower(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
  }
  else // SAME FOR BLACK !!!!!!
  { 
    x2=x;
    y2=y;
    // UP 
    while (1) {
      y2++;
      if (y2>7) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    y2=y;
    // RIGHT
    while (1) {
      x2++;
      if (x2>7) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    x2=x;
    // DOWN
    while (1) {
      y2--;
      if (y2<0) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }
    y2=y;
    // LEFT
    while (1) {
      x2--;
      if (x2<0) break;
      if (position->board[x2][y2] == ' ') 
        continue;
      else if (isupper(position->board[x2][y2])) {
        add(list, newMove(x, y, x2, y2, NO_PROMOTION, 1, c), position);
        break;
      }
      else break;
    }

  }

}


void addQueenCaptures(PPosition position, PMoveList list, int x, int y)
{
  addBishopCaptures(position, list, x, y, 'q');
  addRookCaptures(position, list, x, y, 'q');
}


void addKingCaptures(PPosition position, PMoveList list, int x, int y)
{
  int i, j;
  if (position->whitesMove) {
    for(i=-1; i<=1; i++) {
      for(j=-1; j<=1; j++) {
        if (x+i<0 || x+i>7 || y+j<0 || y+j>7)
          continue; 
        if(islower(position->board[x+i][y+j])) {
          position->board[x][y] = ' ';
          if (!isSquareAttackedByBlack(position, x+i, y+j))
            addWithoutCheck(list,newMove(x,y,(x+i),(y+j),NO_PROMOTION,1,'k'));
          position->board[x][y] = 'K';
        }
      } 
    }
  }
  else {
    for(i=-1; i<=1; i++) {
      for(j=-1; j<=1; j++) {
        if (x+i<0 || x+i>7 || y+j<0 || y+j>7)
          continue; 
        if(isupper(position->board[x+i][y+j])) {
          position->board[x][y] = ' ';
          if (!isSquareAttackedByWhite(position, x+i, y+j))
            addWithoutCheck(list,newMove(x,y,(x+i),(y+j),NO_PROMOTION,1,'k'));
          position->board[x][y] = 'k';
        } 
      } 
    }
  }
}



PMoveList generateCaptures(PPosition position)
{
  int i, j;
  PMoveList list = (PMoveList)malloc(sizeof(TMoveList));
  list->firstFree = 0;
  // WHITE
  if (position->whitesMove) {
    for (i=0; i<8; i++)
      for (j=0; j<8; j++) {
        if (position->board[i][j] == ' ') continue;
        else if (position->board[i][j] == 'P')
          addPawnCaptures(position, list, i, j);
        else if (position->board[i][j] == 'N')
          addKnightCaptures(position, list, i, j);
        else if (position->board[i][j] == 'B')
          addBishopCaptures(position, list, i, j, 'b');
        else if (position->board[i][j] == 'R')
          addRookCaptures(position, list, i, j, 'r');
        else if (position->board[i][j] == 'Q')
          addQueenCaptures(position, list, i, j);
        else if (position->board[i][j] == 'K')
          addKingCaptures(position, list, i, j);
      }
  } 
  // BLACK
  else {
    for (i=0; i<8; i++)
      for (j=0; j<8; j++) {
        if (position->board[i][j] == ' ') continue;
        else if (position->board[i][j] == 'p')
          addPawnCaptures(position, list, i, j);
        else if (position->board[i][j] == 'n')
          addKnightCaptures(position, list, i, j);
        else if (position->board[i][j] == 'b')
          addBishopCaptures(position, list, i, j, 'b');
        else if (position->board[i][j] == 'r')
          addRookCaptures(position, list, i, j, 'r');
        else if (position->board[i][j] == 'q')
          addQueenCaptures(position, list, i, j);
         else if (position->board[i][j] == 'k')
           addKingCaptures(position, list, i, j);
      }
  }
  return list;
}







