/******************************************************************************
 *                                                                            *
 *  Hoplite (c) Frans Ekman 2003                                              *
 *                                                                            *
 *                                                                            *
 *  This file contains the definitions to the search function and some of     *
 *  it's help functions.                                                      *
 *                                                                            *
 *  The search function is a recursive alpha-beta negamax search, that uses   *
 *  transposition tables, killermoves, history heuristics and principal       *
 *  variation collection to improve the move ordering. It also uses null-move *
 *  pruning. See search.c.                                                    *
 *                                                                            *
 *                                                                            *
 *  modified 7.3.2004                                                         *
 *                                                                            *
 *****************************************************************************/


#ifndef SEARCH_H
#define SEARCH_H

#include <string.h>
#include <stdlib.h>
#include "evaluate.h"
#include "hash.h"
#include "killermoves.h"
#include "history.h"


typedef struct timeval Timeval;


typedef struct {
  int cmove; // Number of moves in the line.
  TMove argmove[100];  
} TLine;


typedef struct {
  TMove move;
  int score;
} TMoveScore;


typedef struct {
  TMoveScore moveScore[MAX_POSSIBLE_MOVES];
  int firstFree;
} TMoveScoreList;

/* Help variables for qsort */
extern PPosition helpPosition;
extern int atWhatPlyAreWe;


extern TLine bestLine;
extern int playPVLine;
extern TMoveScoreList moveScoresAtFirstPly;
extern int knowMoveScores;


void printLine(TLine *line);

int cmpMoveScores(const void *table_entry_ptr1, const void *table_entry_ptr2);

int search(PGame game, int depth, int alpha, int beta, TLine *pline, 
           double maxtime, int ply, int maxSearchDepth, int previousNullMove,
           int hasBeenNullMove);


double dtime();

int quiesenceSearch(PGame game, int alpha, int beta);

int fastcmpMoves(const void *table_entry_ptr1, const void *table_entry_ptr2);

int cmpMoves(const void *table_entry_ptr1, const void *table_entry_ptr2);

int getPoints(char c);


#endif
