/******************************************************************************
 *                                                                            *
 *  Hoplite (c) Frans Ekman 2003                                              *
 *                                                                            *
 *  Zobristkey functions implementations.                                     *
 *                                                                            *
 *  modified 24.2.2004                                                        *
 *                                                                            *
 *****************************************************************************/

#include "zobrist.h"

zobristkey zobristKeys[6][2][8][8];
zobristkey zobristWhiteToMove;

zobristkey rand64()
{
  return rand() ^ ((zobristkey)rand() << 15) ^ ((zobristkey)rand() << 30) ^
    ((zobristkey)rand() << 45) ^ ((zobristkey)rand() << 60);
}


void generateZobristKeys()
{
  int piece, color, x, y;

  for (piece=0; piece<6; piece++) 
    for (color=0; color<2; color++) 
      for (x=0; x<8; x++) 
        for (y=0; y<8; y++) 
          zobristKeys[piece][color][x][y] = rand64();

  zobristWhiteToMove = rand64();
}
      

void addPiece(zobristkey *key, char piece, int x, int y)
{
  char c;
  int p;
  int color;

  if (islower(piece)) {
    c = toupper(piece);
    color = BLACK;
  } else {
    color = WHITE;
    c = piece;
  }

  if (c == 'P') 
    p = pawn;
  else if (c == 'N')
    p = knight;
  else if (c == 'B')
    p = bishop;
  else if (c == 'R')
    p = rook;
  else if (c == 'Q')
    p = queen;
  else if (c == 'K')
    p = king;

  *key ^= zobristKeys[p][color][x][y];
}


void switchTurnToMove(zobristkey *key)
{
  *key ^= zobristWhiteToMove;
}


zobristkey getZobristKey(char (*board)[8][8], int whitesmove)
{
  int i, j;
  zobristkey key = 0;

  for (i=0; i<8; i++) {
    for (j=0; j<8; j++) {
      if ((*board)[i][j] != ' ') {
        addPiece(&key, (*board)[i][j], i, j);
      }
    }
  }
  if (whitesmove)
    key ^= zobristWhiteToMove;

  return key;
}


void zobrist_wq_castle(zobristkey *key)
{
  addPiece(key, 'R', 0, 0);
  addPiece(key, 'R', 3, 0);
  addPiece(key, 'K', 4, 0);
  addPiece(key, 'K', 2, 0);
  *key ^= zobristWhiteToMove;
}


void zobrist_wk_castle(zobristkey *key)
{
  addPiece(key, 'R', 7, 0);
  addPiece(key, 'R', 5, 0);
  addPiece(key, 'K', 4, 0);
  addPiece(key, 'K', 6, 0);
  *key ^= zobristWhiteToMove;
}


void zobrist_bq_castle(zobristkey *key)
{
  addPiece(key, 'r', 0, 7);
  addPiece(key, 'r', 3, 7);
  addPiece(key, 'k', 4, 7);
  addPiece(key, 'k', 2, 7);
  *key ^= zobristWhiteToMove;
}


void zobrist_bk_castle(zobristkey *key)
{
  addPiece(key, 'r', 7, 7);
  addPiece(key, 'r', 5, 7);
  addPiece(key, 'k', 4, 7);
  addPiece(key, 'k', 6, 7);
  *key ^= zobristWhiteToMove;
}

  

