/******************************************************************************
 *                                                                            *
 *  Hoplite (c) Frans Ekman 2003                                              *
 *                                                                            *
 *  Zobristkeys are used in the transpositiontables (hashtables). I use 64bit *
 *  Zobristkeys. Those can also be used for comparasion of positions in       *
 *  repetition detection.                                                     *
 *                                                                            *
 *  Zobristkeys are calculated from a position going through the board and    *
 *  for every piece XOR:ing a value corresponding to the piece and its        *
 *  placement. The values are rendomnumbers that are calculated when the      *
 *  program starts.                                                           *
 *                                                                            *
 *  modified 24.2.2004                                                        *
 *                                                                            *
 *****************************************************************************/

#ifndef ZOBRIST_H
#define ZOBRIST_H

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include "def.h"


typedef unsigned long long zobristkey; 


typedef enum { pawn=0, knight=1, bishop=2, rook=3, queen=4, king=5 } pieceType;


/***************************************************
 * [piece][color][x-coordinate][y-coordinate]      *
 ***************************************************/
extern zobristkey zobristKeys[6][2][8][8];


/***************************************************
 * This is XOR:ed if it is whites turn to move.    *
 ***************************************************/
extern zobristkey zobristWhiteToMove;


/***************************************************
 * Generates a 64 bit random number                *
 ***************************************************/
zobristkey rand64();


/***************************************************
 * Generates all the zobrist keys.                 *
 ***************************************************/
void generateZobristKeys();


/***************************************************
 * XORs the key with the zobrist key representing  *
 * the piece (also used for removing a piece.      *
 ***************************************************/
void addPiece(zobristkey *key, char piece, int x, int y);


/***************************************************
 * XORs the key with the zobrist key representing  *
 * whites move.                                    *
 ***************************************************/
void switchTurnToMove(zobristkey *key);


/***************************************************
 * Return a zobristkey for the bosition.           *
 ***************************************************/
zobristkey getZobristKey(char (*board)[8][8], int whitesmove);


/***************************************************
 * Update a zobristkey after a castle move.        *
 ***************************************************/
void zobrist_wq_castle(zobristkey *key);
void zobrist_wk_castle(zobristkey *key);
void zobrist_bq_castle(zobristkey *key);
void zobrist_bk_castle(zobristkey *key);



#endif


